
// Jan Woetzel 03/2005

#include "FileHandling.hh"

// STD
#include <iostream>
#include <fstream>

// BIAS
#include "Base/Debug/Error.hh"



using namespace std;
//using namespace BIAS:


// JW
bool BIAS::FileReadable( const std::string & s)
{
  ifstream ifs( s.c_str() );
  // ok, if file could be opended and is redable
  bool ok = ifs.good();
  ifs.close();
  return ok;
}



void BIAS::SplitName(const std::string & fullname,
		     string & dir,
		     string & base,
		     string & ext )
{ 
  // find from right 
  // expr: dir/subdir/base.ext
  string::size_type posL = fullname.rfind('/'); // separator for directory
  string::size_type posR = fullname.rfind('.'); // separator for suffix
  
  // which separators were found ?
  bool foundDir = (posL!=std::string::npos);
  bool foundExt = (posR!=std::string::npos);
  
  if ( foundDir && foundExt ) {
    // found path and extension. base is inner part
    dir  = fullname.substr(0, posL+1 ); // start, length
    base = fullname.substr(posL+1, (posR-posL-1)  );
    ext  = fullname.substr(posR, (fullname.size()-posR)  );    
    
  } else if ( foundDir && !foundExt) {
    dir  = fullname.substr(0, posL+1 ); // start, length
    base = fullname.substr(posL+1, (fullname.size()-posL) );
    ext  = ""; // no extnesion found.
    
  } else if ( !foundDir && foundExt) {
    dir  = "";
    base = fullname.substr(0, posR);
    ext  = fullname.substr(posR, (fullname.size()-posR) );    
    
  } else {    
    // is laready a basename. 
    BIASERR("could not split path. delimiters not found. ");
    dir="";
    base=fullname;
    ext="";
  };

  /*
    cout<<"DBG "<<endl
    <<"fullname = "<<fullname<<endl
    <<"dir      = "<<dir<<endl
    <<"base     = "<<base<<endl
    <<"ext      = "<<ext<<endl;
  */
}


std::string BIAS::Basename(const std::string & fullname){
  string dir, base, ext;
  BIAS::SplitName(fullname,
		  dir, base, ext );
  return base;
}


std::string BIAS::Directory(const std::string & fullname){
  string dir, base, ext;
  BIAS::SplitName(fullname,
		  dir, base, ext );
  return dir;
}




void BIAS::LogCommandLine( int argc,char** argv, std::ostream & os){
  for (int i=0; i<argc; i++){
    os<<argv[i]<<" "<<endl;
  }
  os<<endl;
  os<<"argc="<<argc<<endl;
  for (int i=0; i<argc; i++){
    os<<"argv["<<i<<"]="<<argv[i]<<endl;
  }
  os<<flush;
}

void BIAS::LogCommandLine( int argc,char** argv, const std::string & filename){
  ofstream ofs;
  ofs.open(filename.c_str()
	   //, ios::app 
	   );
  if (!ofs.good()) {
    BIASERR("could not open "<<filename<<" for LogCommandLine.");
    LogCommandLine(argc, argv, std::cout);
  } else {
    LogCommandLine(argc, argv, ofs);    
  };
  ofs.close();
}



// return leading zero std::string of number JW 05/2003
std::string 
BIAS::LeadingZeroString( const int & n, 
			 const unsigned int & digits ) 
{ 
  char bufC[10]; //  to convert the integer into a string
  // s#thsi switch is not pretty but works... 
  switch (digits) {
  case 1:
    sprintf( bufC,"%01i", n);
    break;
  case 2:
    sprintf( bufC,"%02i", n);
    break;
  case 3:
    sprintf( bufC,"%03i", n);
    break;
  case 4:
    sprintf( bufC,"%04i", n);
    break;
  case 5:
    sprintf( bufC,"%05i", n);
    break;
  case 6 :
    sprintf( bufC,"%06i", n);
    break;
  case 7:
    sprintf( bufC,"%07i", n);
    break;
  case 8:
    sprintf( bufC,"%08i", n);
    break;
 case 9:
    sprintf( bufC,"%09i", n);
    break;
 case 10:
   sprintf( bufC,"%010i", n);
   break;

  default:
    BIASERR("error: overflow for digits="<<digits);
    sprintf( bufC,"%010i", n);
  };
  std::string str;
  str = bufC;
  return str;
}
