/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#ifndef __DEBUG_SIMPLE_HH__
#define __DEBUG_SIMPLE_HH__

/** 
    @brief Macros for output instead of direct cout<<
    Can be switched on or off globally
    
    The significant difference to [MIP/BIAS]Debug is that they need additional class members and levels
    COUT: either on or off (global)
    Use BIAS/Base/Debug.hh for more granular debugging/outputs. 
    @author Jan Woetzel
    @status beta (07/03/2003) added std:: and tested
    alpha (02/28/2002)
**/

// for std::cout / std::cerr
#include <iostream>


// do COUT outputs?, default is 'on'
#ifndef COUT_ONOFF
#  define COUT_ONOFF 1
#endif

// which destination for COUT ?
#ifndef COUT_DEST
#  define COUT_DEST std::cout
#endif


// ------ define the macros ------------------------------------------------------

#if COUT_ONOFF
// -- COUT's argument is an ostream to be printed
#  define COUT(output) \
     COUT_DEST<<output;
#else
// -- do nothing
#  define COUT(output)
#endif



#endif // do not add code after this line
