/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include "MyDebug.hh"

#include <Base/Debug/Debug.hh>
#include <Base/Debug/Error.hh>
#include <fstream>

using namespace BIAS;
using namespace std;

int main(int argc, char *argv[])
{
  ExampleDebug instance1;
  ExampleDebug instance2;



  /////////////////////////////////////////////////////////////
  // using local per-instance debuglevel
  ////////////////////////////////////////////////////////////

  // set the debuglevel for instance1
  instance1.SetDebugLevel(instance1.GetDebugLevel()|EXAMPLEDEBUG_TESTFLAG);
  instance1.SetDebugLevel(instance1.GetDebugLevel()|EXAMPLEDEBUG_TESTFLAG2);

  // this instance is with debug information
  cout << "\ninstance1 with 'local' debug information:\n" << endl;
  instance1.func();

  // this instance is without debug information
  cout << "\ninstance2 without 'local' debug information:\n" << endl;
  instance2.func();
  


  //////////////////////////////////////////////////////////////
  //  using the global debug level
  /////////////////////////////////////////////////////////////
  
  // the global debg level
  instance1.SetGlobalDebugLevel(150);
  
  cout << "\ninstance1 with 'global' debug information:\n" << endl;
  instance1.func();
  
  cout << "\ninstance2 with 'global' debug information:\n" << endl;
  instance2.func();
  cout << "\nthe constructor should be called after the global\n"
       <<" debuglevel 'BIASGlobalDebugLevel' is set:\n" << endl;
  ExampleDebug instance3;
  instance3.func();

  cout << "\nor the reset must be done by hand:\n" << endl;
  instance2.ResetDebugLevelFromGlobal();
  instance2.func();

  /////////////////////////////////////////////////////////////
  //    redirecting debug output
  ///////////////////////////////////////////////////////////////

  cout << "\nnow redirect the debug output to file 'debug.txt':\n" << endl;
  ofstream os("debug.txt");
  if (!os.good())
    BIASERR("error opening debug.txt");
  instance1.SetDebugStream(os);
  instance1.func();

  os.close();

  cout << "\nnow use cerr again:\n" << endl;
  instance1.SetDebugStream(cerr);
  instance1.func();
  
  return 0;
}
