/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include "AffineTransf.hh"

using namespace BIAS;
using namespace std;

AffineTransf::AffineTransf() :
  Matrix3x4<double>(), R_(), C_()
{}

AffineTransf::~AffineTransf() 
{}

HomgPoint3D AffineTransf::operator*(HomgPoint3D& argv)
{
  HomgPoint3D res;
  Mult(argv, res);
  return res;
}

void 
AffineTransf::Mult(Vector4<double>& argv, Vector4<double>& destv)
{
  destv[0] =
    v_[0] * argv[0]
    +v_[1] * argv[1]
    +v_[2] * argv[2]
    +v_[3] * argv[3];
  destv[1] =
    v_[4] * argv[0]
    +v_[5] * argv[1]
    +v_[6] * argv[2]
    +v_[7] * argv[3];
  destv[2] =
    v_[8]  * argv[0]
    +v_[9]  * argv[1]
    +v_[10] * argv[2]
    +v_[11] * argv[3];
  // fake last row, affine transf has normaly size 4x4
  destv[3] = argv[3];
}

AffineTransf& 
AffineTransf::operator=(const AffineTransf& arg)
{
  double *p = arg.GetData();
  memcpy(v_, p, 12*sizeof(double));
  R_ = arg.GetR();
  C_ = arg.GetC();
  return *this;
}


void 
AffineTransf::SetAsRotationMatrix(Matrix3x3<double>& R)
{
  double *p = R.GetData();
  v_[0] = p[0]; v_[1] = p[1]; v_[2] = p[2];
  v_[4] = p[3]; v_[5] = p[4]; v_[6] = p[5];
  v_[8] = p[6]; v_[9] = p[7]; v_[10] = p[8];
  v_[3] = v_[7] = v_[11] = 0.0;
}
