#ifndef _IMAGEATTRIBUTES_H_INCLUDED_
#define _IMAGEATTRIBUTES_H_INCLUDED_


#include "bias_config.h"

#include "ExifTag.hh"
#include <vector>
#include <iostream>

/// return code for success:
#define BIAS_IMAGEATTRIBUTES_SUCCESS 0

  
#ifdef BIAS_HAVE_IMAGEMAGICKLIB
#  ifdef WIN32
#    pragma warning( disable : 4251)  // JW
#  endif
#  include <Magick++.h>
#endif
/// \todo Avoid inclusion of Magick++ header and use forward declaration here to be able to output runtime error on usage (JW).


// Begin namespace BIAS
namespace BIAS {
  
  /** @class ImageAttributes
      @todo Add Image Attributes write functionality to write meta data directly to images EXIF Tags, comments etc. 
      @brief contains all atribute info and values of e.g. a file.
      @author Jan Woetzel 10/2004 */
  class ImageAttributes : public std::vector< ExifTag > {
  public:
    ~ImageAttributes(){};
    ImageAttributes(){};
       
    /** @brief main entry routine to read all EXIF tags from the header of image file.
	@return  BIAS_IMAGEATTRIBUTES_SUCCESS if OK.
	@param filename image to ping and read header from.
	@author Jan Woetzel */
    int Read(const std::string & filename);
    
    /** @return  BIAS_IMAGEATTRIBUTES_SUCCESS if reading was OK, negative number in case of error.
	@param filename image file to read attributes form header.
	@author Jan Woetzel */
    int ReadMagickPP(const std::string & filename);
    
#ifdef BIAS_HAVE_IMAGEMAGICKLIB
    /** @brief direct interface to Magick::IMage to be able to read the XIF tags of an already opened image. 
	@return  BIAS_IMAGEATTRIBUTES_SUCCESS;BIAS_EXIFTAGTABLE_SUCCESS 
	@param filename image to ping and read header from.
	@author Jan Woetzel */
    int ReadMagickPP(Magick::Image & img);
#endif
    
    
    /** initialize me vector with all known EXIF tags */
    void InitExifTags();
    
    /** print teh content of this to os 
	@param printAll true prints all tags, false only those whose value is not empty.
     */
    std::ostream& Print(std::ostream& os=std::cout,
			const bool printAll=false) const;
    
  }; // end of class
} // end namespace


// ------------------

/** @brief output operator for ExifTable 
    @author Jan Woetzel */
std::ostream& operator<<(std::ostream& os, 
			 const BIAS::ImageAttributes &attr);


#endif
