/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include "Matrix2x2.hh"

using namespace BIAS;
using namespace std;

// template<class T> 
// Matrix2x2<T>::Matrix2x2(const Matrix<T> & A) 
// : Matrix<T>(2, 2)
// {
//   if (A.num_rows() != 2 || A.num_cols() != 2){
//     BIASERR("try to cast from Matrix with inavlid size "<<A);
//   } else {
//     register T *dataP = A.GetData();
//     memcpy(v_, dataP, 4*sizeof(T));
//   }
// }

template<class T>
Matrix2x2<T>::Matrix2x2<T>(const TNT::Matrix<T>& mat) : Matrix<T>(mat)
{
  if ((mat.num_rows() != 2) || (mat.num_cols() != 2)) {
    BIASERR("wrong matrix size "<<mat.num_rows()<<"x"
           << mat.num_cols());
  }
}


template<class T> 
Matrix2x2<T>::~Matrix2x2() {}

template<class T>
Matrix2x2<T>& Matrix2x2<T>::operator= (const Matrix2x2<T>& mat) 
{
  register T *matP = mat.GetData();
  register T *destP = this->GetData();
  memcpy(destP, matP, 4*sizeof(T));
  return *this;
}

template<class T>
Matrix2x2<T>& Matrix2x2<T>::newsize(int rows, int cols) 
{
    BIASERR("does not make sense with fixed size matrix: "<<rows<<" "<<cols);
  return *((Matrix2x2 *)NULL);
}



template<class T>
int Matrix2x2<T>::Invert(Matrix2x2<T>& result) const
{
  T den=(*this)[0][0]*(*this)[1][1]-(*this)[0][1]*(*this)[1][0];
  //if (den==0.0) std::cerr << *this << std::endl << den << std::endl;
  if (den==0.0) return -1;
  BIASASSERT(den!=0.0);

  result[0][0]=(*this)[1][1]/den;
  result[1][0]=-(*this)[1][0]/den;
  result[0][1]=-(*this)[0][1]/den;
  result[1][1]=(*this)[0][0]/den;
  return 0;
}






