/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include "Vector.hh"

using namespace BIAS;
using namespace std;

template<class T> 
Vector<T>::~Vector() 
{}

/*
template<class T> 
Vector<T>::Vector(const Vector2<T>& v) 
{
    newsize(2);			// set this vector size to size of argument vector
    GetData()[0] = v[0];	// set elements
    GetData()[1] = v[1];
}

template<class T> 
Vector<T>::Vector(const Vector3<T>& v) 
{
    newsize(3);			// set this vector size to size of argument vector
    GetData()[0] = v[0];	// set elements
    GetData()[1] = v[1];
    GetData()[2] = v[2];
}

template<class T> 
Vector<T>::Vector(const Vector4<T>& v) 
{
    newsize(4);			// set this vector size to size of argument vector
    GetData()[0] = v[0];	// set elements
    GetData()[1] = v[1];
    GetData()[2] = v[2];
    GetData()[3] = v[3];
}
*/

template<class T>
Vector<T>& Vector<T>::operator=(const TNT::Vector<T> &vec) 
{
    TNT::Vector<T>::operator = (vec);	// call the operator of the base class
    return *this;
}

template<class T> 
void Vector<T>::Fill(const T & scalar) 
{
    TNT::Vector<T>::operator=(scalar);
}


template <class T>
Matrix<T> Vector<T>::OuterProduct(const Vector<T> &v) const {
  Matrix<T> mat(this->size(),v.size());
  T rVal;
  for (int row=0; row<this->size(); row++) {
    rVal = (*this)[row];
    for (int col=0; col<v.size(); col++) 
      mat[row][col] = rVal * v[col];
  }
  return mat;
}

template <class T>
void Vector<T>::KroneckerProduct(const Vector<T> &arg, Vector<T>& dst) const
{
  const unsigned s1=this->size(), s2=arg.size();
  unsigned l=0;
  dst.newsize(s1*s2);
  for (register unsigned i=0; i<s1; i++){
    for (register unsigned k=0; k<s2; k++){
      dst[l]=(*this)[i]*arg[k];
      l++;
    }
  }
}



template <class T>
bool Vector<T>::IsZero() const {
  int i=this->size();
  // return false when the first non-zero element is found.
  while (i) {
    i--;
    if (this->v_[i] != (T)0) return false;
  };
  return true; // all elementa are zero
}
    
