##
## set additional OS/compiler specific flags
## 
## Jan-Felix Woelk and Jan Woetzel 07/2004
##
## TODO: Avoid FORCE usage 
##  - to be able to edit the flags in the GUI
##  - to allow Test script to try flags
## 

IF (UNIX)
  ## cmakes default debug build does not set -Wall, unfortunately
  ## We really need FORCE to do it (!!!) 
  ## FIXME, TODO: Changing ccmake gui entries has no effect due to FORCE (JW)
  ## TODO: instead of using FORCE an Cache file could be used to pre-initialize the values.
  ##

  ## edit "Standard build" flags provided by cmake:
  SET(CMAKE_CXX_FLAGS_DEBUG "-Wall -g -pedantic" 
    CACHE STRING "Debug builds CMAKE CXX flags (forced)" FORCE )
  SET(CMAKE_C_FLAGS_DEBUG "-Wall -g -pedantic" 
    CACHE STRING "Flags used by the compiler during debug builds." FORCE )  

  ## debug without pedantic
  SET(CMAKE_CXX_FLAGS_DEBUG2 "-Wall -g " 
    CACHE STRING "Debug2 builds CMAKE CXX flags (forced)" FORCE )
  SET(CMAKE_C_FLAGS_DEBUG2 "-Wall -g " 
    CACHE STRING "Flags used by the compiler during Debug2 builds." FORCE ) 

  SET(CMAKE_CXX_FLAGS_RELEASE "-O3 -Wall -pedantic" 
    CACHE STRING "Flags used by the compiler during release builds." FORCE )
  SET(CMAKE_C_FLAGS_RELEASE "-O3 -Wall -pedantic" 
    CACHE STRING "Flags used by the compiler during release builds." FORCE )

  SET(CMAKE_CXX_FLAGS_RELWITHDEBINFO "-O3 -Wall -g -pedantic" 
    CACHE STRING "Flags used by the compiler during release builds." FORCE )
  SET(CMAKE_C_FLAGS_RELWITHDEBINFO "-O3 -Wall -g -pedantic" 
    CACHE STRING "Flags used by the compiler during release builds." FORCE )  
  
  SET(CMAKE_CXX_FLAGS_RELEASEO2 "-O2 -Wall -pedantic" 
    CACHE STRING "Flags used by the compiler during release builds." FORCE )
  SET(CMAKE_C_FLAGS_RELEASEO2 "-O2 -Wall -pedantic" 
    CACHE STRING "Flags used by the compiler during release builds." FORCE )

  ## add our own architecture optimized builds:
  SET(CMAKE_CXX_FLAGS_P4 "-O3 -march=pentium4 -Wall -pedantic" 
    CACHE STRING "Flags used by the compiler during release builds.")
  SET(CMAKE_C_FLAGS_P4 "-O3 -march=pentium4 -Wall -pedantic" 
    CACHE STRING "Flags used by the compiler during release builds." )

  ## add our own architecture optimized builds:
  SET(CMAKE_CXX_FLAGS_P4SSE "-O3 -march=pentium4 -mcpu=pentium4 -msse -msse2 -mfpmath=sse -minline-all-stringops -Wall -pedantic" 
    CACHE STRING "Flags used by the compiler during release builds.")
  SET(CMAKE_C_FLAGS_P4SSE "-O3 -march=pentium4 -mcpu=pentium4 -msse -msse2 -mfpmath=sse -minline-all-stringops -Wall -pedantic" 
    CACHE STRING "Flags used by the compiler during release builds." )

  SET(CMAKE_CXX_FLAGS_P4DEB "-O3 -march=pentium4 -Wall -g -pedantic" 
    CACHE STRING "Flags used by the compiler during release builds.")
  SET(CMAKE_C_FLAGS_P4DEB "-O3 -march=pentium4 -Wall -g -pedantic" 
    CACHE STRING "Flags used by the compiler during release builds." )

  SET(CMAKE_CXX_FLAGS_P4PROF "-O3 -march=pentium4 -g -pg -Wall -pedantic" 
    CACHE STRING "Flags used by the compiler during profiling builds.")
  SET(CMAKE_C_FLAGS_P4PROF "-O3 -march=pentium4 -g -pg -Wall -pedantic" 
    CACHE STRING "Flags used by the compiler during profiling builds.")
  SET(CMAKE_EXE_LINKER_FLAGS_P4PROF "-pg" 
    CACHE STRING "Flags used by the linker during profiling builds.")
  SET(CMAKE_MODULE_LINKER_FLAGS_P4PROF "-pg" 
    CACHE STRING "Flags used by the linker during profiling builds.")

  SET(CMAKE_CXX_FLAGS_P4SSEPROF "-O3 -march=pentium4 -mcpu=pentium4 -msse -msse2 -mfpmath=sse -minline-all-stringops -g -pg -Wall -pedantic" 
    CACHE STRING "Flags used by the compiler during profiling builds.")
  SET(CMAKE_C_FLAGS_P4SSEPROF "-O3 -march=pentium4 -mcpu=pentium4 -msse -msse2 -mfpmath=sse -minline-all-stringops -g -pg -Wall -pedantic" 
    CACHE STRING "Flags used by the compiler during profiling builds.")
  SET(CMAKE_EXE_LINKER_FLAGS_P4SSEPROF "-pg" 
    CACHE STRING "Flags used by the linker during profiling builds.")
  SET(CMAKE_MODULE_LINKER_FLAGS_P4SSEPROF "-pg" 
    CACHE STRING "Flags used by the linker during profiling builds.")

  SET(CMAKE_CXX_FLAGS_P4SSEDEB "-O3 -march=pentium4 -mcpu=pentium4 -msse -msse2 -mfpmath=sse -minline-all-stringops -g -Wall -pedantic" 
    CACHE STRING "Flags used by the compiler during profiling builds.")
  SET(CMAKE_C_FLAGS_P4SSEDEB "-O3 -march=pentium4 -mcpu=pentium4 -msse -msse2 -mfpmath=sse -minline-all-stringops -g -Wall -pedantic" 
    CACHE STRING "Flags used by the compiler during profiling builds.")

  SET(CMAKE_CXX_FLAGS_ATHLON "-O3 -march=athlon -Wall -pedantic" 
    CACHE STRING "Flags used by the compiler during release builds.")
  SET(CMAKE_C_FLAGS_ATHLON "-O3 -march=athlon -Wall -pedantic" 
    CACHE STRING "Flags used by the compiler during release builds.")

  ## Pentium4 with prescott core that supports sse3  - JW
  SET(CMAKE_CXX_FLAGS_PRESCOTT "-O3 -march=prescott -mcpu=prescott -mmmx -msse -msse2 -msse3 -mfpmath=sse -minline-all-stringops -Wall" 
    CACHE STRING "Flags used by the compiler during PRESCOTT optimized builds.")
  SET(CMAKE_C_FLAGS_PRESCOTT   "-O3 -march=prescott -mcpu=prescott -mmmx -msse -msse2 -msse3 -mfpmath=sse -minline-all-stringops -Wall" 
    CACHE STRING "Flags used by the compiler during PRESCOTT optimized builds." )


# hide them (JW)
MARK_AS_ADVANCED(
  CMAKE_CXX_FLAGS_P4
  CMAKE_C_FLAGS_P4
  CMAKE_CXX_FLAGS_P4DEB
  CMAKE_C_FLAGS_P4DEB
  CMAKE_CXX_FLAGS_P4SSE
  CMAKE_C_FLAGS_P4SSE
  CMAKE_CXX_FLAGS_P4PROF
  CMAKE_C_FLAGS_P4PROF
  CMAKE_EXE_LINKER_FLAGS_P4PROF
  CMAKE_MODULE_LINKER_FLAGS_P4PROF
  CMAKE_CXX_FLAGS_P4SSEPROF
  CMAKE_C_FLAGS_P4SSEPROF
  CMAKE_EXE_LINKER_FLAGS_P4SSEPROF
  CMAKE_MODULE_LINKER_FLAGS_P4SSEPROF
  CMAKE_CXX_FLAGS_P4SSEDEB
  CMAKE_C_FLAGS_P4SSEDEB
  CMAKE_CXX_FLAGS_ATHLON
  CMAKE_C_FLAGS_ATHLON
  CMAKE_CXX_FLAGS_RELEASEO2 
  CMAKE_C_FLAGS_RELEASEO2 
  CMAKE_CXX_FLAGS_DEBUG2
  CMAKE_C_FLAGS_DEBUG2
  CMAKE_CXX_FLAGS_PRESCOTT
  CMAKE_C_FLAGS_PRESCOTT
)

ENDIF(UNIX)
## ---------------------------------------------------
IF (WIN32)

  ## Pentium4 with prescott core that supports sse3  - JW
  ## TODO: add sse3 support (only Visual Studio 2005  /arch:SSE3 ?)
  SET(CMAKE_CXX_FLAGS_PRESCOTT "/MD /Y3 /O2 /G7 /Ot /nologo /D NDEBUG /W3 /arch:SSE /arch:SSE2"
    CACHE STRING "Flags used by the compiler during release builds.")
  SET(CMAKE_C_FLAGS_PRESCOTT   "/MD /Y3 /O2 /G7 /Ot /nologo /D NDEBUG /W3 /arch:SSE /arch:SSE2"
    CACHE STRING "Flags used by the compiler during release builds." )

  ##
  ## set highest warning level (JW)
  ##
  IF(CMAKE_BUILD_TOOL MATCHES "(msdev|devenv|nmake)")  
    # Use the highest warning level for visual studio.
    # SET(CMAKE_CXX_WARNING_LEVEL 4  CACHE STRING "warning level" FORCE)
    SET(CMAKE_CXX_WARNING_LEVEL 3  CACHE STRING "warning level" FORCE)
    # SET(CMAKE_CXX_WARNING_LEVEL 2  CACHE STRING "warning level" FORCE)
    # SET(CMAKE_CXX_WARNING_LEVEL 1  CACHE STRING "warning level" FORCE)
    
    IF(CMAKE_CXX_FLAGS MATCHES "/W[0-4]")
      STRING(REGEX REPLACE "/W[0-4]" "/W${CMAKE_CXX_WARNING_LEVEL}" 
        CMAKE_CXX_FLAGS "${CMAKE_CXX_FLAGS}")
    ELSE(CMAKE_CXX_FLAGS MATCHES "/W[0-4]")
      SET(CMAKE_CXX_FLAGS "${CMAKE_CXX_FLAGS} /W${CMAKE_CXX_WARNING_LEVEL}")
    ENDIF(CMAKE_CXX_FLAGS MATCHES "/W[0-4]")
  ENDIF(CMAKE_BUILD_TOOL MATCHES "(msdev|devenv|nmake)")

  ##
  ## avoid clash between LIBC/MSVCRT (JW)
  ##
  #(JW) SET(CMAKE_EXE_LINKER_FLAGS "${CMAKE_EXE_LINKER_FLAGS} /NODEFAULTLIB:\"LIBC.lib\" /NODEFAULTLIB:\"MSVCRT.lib\"")
  SET(CMAKE_EXE_LINKER_FLAGS "${CMAKE_EXE_LINKER_FLAGS} /NODEFAULTLIB:\"LIBC.lib\"")


MARK_AS_ADVANCED(  
  CMAKE_CXX_FLAGS_PRESCOTT
  CMAKE_C_FLAGS_PRESCOTT
)
ENDIF(WIN32)

