## This file generates the bias library config shell script: "bias-config"
## from bias-config.in (in BIAS_SOURCE_DIR) 
##
## -------------------------------------------------------------------------
## Jan-Friso Evers-Senne evers@mip.informatik.uni-kiel.de
## Jan Woetzel 06/2004 jw@mip.informatik.uni-kiel.de
## -------------------------------------------------------------------------


IF (UNIX)
  SET(FILE_BIAS_CONFIG_SCRIPT_IN "${BIAS_SOURCE_DIR}/bias-config.in")
  SET(FILE_BIAS_CONFIG_SCRIPT "${BIAS_BINARY_DIR}/bias-config")
  
  # where to grab this from (different for installed/notinstall case)
  SET(BIAS_PREFIX "${CMAKE_INSTALL_PREFIX}")

  #
  # extra compiler flags rpaths:
  #
  SET(BIAS_LD_FLAGS "-Wl,-rpath,${BIAS_PREFIX}/lib")
  IF(USE_OPENCV AND HAVE_OPENCV)
    SET(BIAS_LD_FLAGS "${BIAS_LD_FLAGS}:${OPENCV_LINK_DIRECTORIES}")
  ENDIF(USE_OPENCV AND HAVE_OPENCV)
  
  
#  # ==============================================
#  #  Recursive dependency solving via *_LIB_DEPENDS
#  #  is correct but too slow, currently 
#  #  because of wide recursive foreach tree. (JW)
#  #  keep the old "handmade" IF USE_*  for now. (JW)
#  #  TODO, FIXME, HACK
#  # ==============================================
#  #
#  # Append value to list in case value is not yet in var (JW)
#  MACRO(LIST_APPEND_UNIQUE list value)
#    SET(LIST_ADD_UNIQUE_FLAG 0)
#    # check if value is already in var 
#    FOREACH(i ${${list}})
#      IF ("${i}" MATCHES "${value}")
#        SET(LIST_ADD_UNIQUE_FLAG 1)
#      ENDIF("${i}" MATCHES "${value}")
#    ENDFOREACH(i)
#    IF(NOT LIST_ADD_UNIQUE_FLAG)
#      # append
#      SET(${list} ${${list}} ${value})
#    ENDIF(NOT LIST_ADD_UNIQUE_FLAG)
#  ENDMACRO(LIST_APPEND_UNIQUE)  
#  
#  # recursively compute all Dependencies PARENT depends on directly or implicitly. 
#  MACRO(EXTERNAL_DEPS_REC parent allDeps)
#    FOREACH(child ${${parent}_LIB_DEPENDS})
#      #IF (NOT "${child}" MATCHES "BIAS")
#      LIST_APPEND_UNIQUE(${allDeps} ${child})
#      #SET(${allDeps} ${${allDeps}} ${child})
#      #ENDIF (NOT "${child}" MATCHES "BIAS")
#      ## recurse: 
#      EXTERNAL_DEPS_REC(${child} ${allDeps})
#    ENDFOREACH(child)
#  ENDMACRO(EXTERNAL_DEPS_REC)
#  
#  #EXTERNAL_DEPS_REC("bias" BIAS_EXTERN_LIBRARIES)
#  MESSAGE("DBG BIAS_EXTERN_LIBRARIES=${BIAS_EXTERN_LIBRARIES}")

  IF(USE_FFTW3)
    SET(BIAS_EXTERN_LIBRARIES  "${BIAS_EXTERN_LIBRARIES} ${FFTW3_LIBRARIES}")
  ENDIF(USE_FFTW3)
  IF(GTK_FOUND)
    SET(BIAS_EXTERN_LIBRARIES "${BIAS_EXTERN_LIBRARIES} ${GTK_LIBRARIES}")
  ENDIF(GTK_FOUND)
  IF(USE_OPENGL AND GTK_GL_FOUND)
    SET(BIAS_EXTERN_LIBRARIES ${BIAS_EXTERN_LIBRARIES}  ${OPENGL_LIBRARIES})
  ENDIF(USE_OPENGL AND GTK_GL_FOUND)
  IF(USE_OPENCV)
    SET(BIAS_LD_FLAGS "${BIAS_LD_FLAGS}:${OPENCV_LINK_DIRECTORIES}")
    SET(BIAS_EXTERN_LIBRARIES ${BIAS_EXTERN_LIBRARIES} ${OPENCV_LIBRARIES})
    #MESSAGE("DBG OPENCV_LINK_DIRECTORIES=${OPENCV_LINK_DIRECTORIES}")
  ENDIF(USE_OPENCV)
  IF(USE_IMLIB)
    SET(BIAS_EXTERN_LIBRARIES ${BIAS_EXTERN_LIBRARIES} ${IMLIB_LIBRARIES})
  ENDIF(USE_IMLIB)
  IF(USE_LIBXML2)
    SET(BIAS_EXTERN_LIBRARIES ${BIAS_EXTERN_LIBRARIES} ${LIBXML2_LIBRARIES})
  ENDIF(USE_LIBXML2)
  # add required libs (Lapack, blas)
  SET(BIAS_EXTERN_LIBRARIES  ${BIAS_EXTERN_LIBRARIES}  ${BLAS_LIBRARIES} ${LAPACK_LIBRARIES})
  
  
  ## generate old-style bias-config shell script 
  ## to be used just by NON-CMAKIFIED user projects. 
  ## See PackageConfig/BIASConfig for CMAKE user projects
  
  SET(BIAS_CONFIG_SCRIPT_C_FLAGS "")
  SET(BIAS_CONFIG_SCRIPT_C_FLAGS "${BIAS_CONFIG_SCRIPT_C_FLAGS} -I${CMAKE_INSTALL_PREFIX}/include")
  SET(BIAS_CONFIG_SCRIPT_C_FLAGS "${BIAS_CONFIG_SCRIPT_C_FLAGS} -I${CMAKE_INSTALL_PREFIX}/include/BIAS")

  FOREACH(INCDIR ${BIAS_EXTERN_INCLUDE_DIR})
    SET(BIAS_CONFIG_SCRIPT_C_FLAGS "${BIAS_CONFIG_SCRIPT_C_FLAGS} -I${INCDIR}")
  ENDFOREACH(INCDIR)

  # prefix with BIAS own installed include paths
  SET(BIAS_CONFIG_SCRIPT_C_FLAGS "-I${BIAS_PREFIX}/include -I${BIAS_PREFIX}/include/BIAS ${BIAS_CONFIG_SCRIPT_C_FLAGS}")
  # MESSAGE("DBG BIAS_CONFIG_SCRIPT_CFLAGS: ${BIAS_CONFIG_SCRIPT_C_FLAGS}")
  
  
  ##
  ## add "-l" to "real" library names  
  ## skip *.so  and all back-quoted shell scripts
  ##
  SET(BIAS_CONFIG_SCRIPT_EXTERN_LIBRARIES "")
  FOREACH(LIB ${BIAS_EXTERN_LIBRARIES})
    # do not prefix back-quited expressions, e.g/ `/bin/foo-config`
    IF (NOT ${LIB} MATCHES "[`].+[`]")
      IF (NOT ${LIB} MATCHES ".+[.][s][o]")
	IF (NOT ${LIB} MATCHES "[-][l].+")
	  
	  # OK, this a 'real' library to be prefixed: 
	  # add prefix and convert semicolon-separated vector to space-separtaed string 
	  SET(BIAS_CONFIG_SCRIPT_EXTERN_LIBS "${BIAS_CONFIG_SCRIPT_EXTERN_LIBS} -l${LIB}")
	  
	ELSE (NOT ${LIB} MATCHES "[-][l].+")
	  # MESSAGE("DBG skipping  already -l prefixed LIB=${LIB}")
	  SET(BIAS_CONFIG_SCRIPT_EXTERN_LIBS "${BIAS_CONFIG_SCRIPT_EXTERN_LIBS} ${LIB}")	
	ENDIF (NOT ${LIB} MATCHES "[-][l].+")
      ELSE (NOT ${LIB} MATCHES ".+[.][s][o]")
	#MESSAGE("DBG skipping  prefixing of .so LIB=${LIB}")
	SET(BIAS_CONFIG_SCRIPT_EXTERN_LIBS "${BIAS_CONFIG_SCRIPT_EXTERN_LIBS} ${LIB}")	
      ENDIF (NOT ${LIB} MATCHES ".+[.][s][o]")      
    ELSE (NOT ${LIB} MATCHES "[`].+[`]")
      #MESSAGE("DBG skipping prefixing of back-quoted LIB=${LIB}")
      SET(BIAS_CONFIG_SCRIPT_EXTERN_LIBS "${BIAS_CONFIG_SCRIPT_EXTERN_LIBS} ${LIB}")
    ENDIF (NOT ${LIB} MATCHES "[`].+[`]")
  ENDFOREACH(LIB)
  #MESSAGE("DBG  BIAS_CONFIG_SCRIPT_EXTERN_LIBS=${BIAS_CONFIG_SCRIPT_EXTERN_LIBS}")
  
  ## DEPRECATED (JW) 
  # replace all ";" with " "
  #STRING(REGEX REPLACE "[;]" " " BIAS_CONFIG_SCRIPT_EXTERN_LIBRARIES "${BIAS_EXTERN_LIBRARIES}" )
  
  # prepend with BIAS own lib dir
  SET(BIAS_CONFIG_SCRIPT_EXTERN_LIBS  "-L${BIAS_PREFIX}/lib ${BIAS_CONFIG_SCRIPT_EXTERN_LIBS}")

  CONFIGURE_FILE(${FILE_BIAS_CONFIG_SCRIPT_IN} ${FILE_BIAS_CONFIG_SCRIPT})

  ## doesnt work because of missing target (JW)
  #ADD_CUSTOM_COMMAND( TARGET ${FILE_BIAS_CONFIG_SCRIPT}
  #  POST_BUILD
  #  COMMAND chmod
  #  ARGS 755 ${FILE_BIAS_CONFIG_SCRIPT}
  #  COMMENT "setting exe flag for ${FILE_BIAS_CONFIG_SCRIPT} (JW)")


ELSE(UNIX)
  IF(WIN32)
	# OK, do nothing for now. (JW)
  ELSE(WIN32)
	MESSAGE("GenerateConfigScript works only on Unix/Linux/WIN32, sorry.")
  ENDIF(WIN32)
ENDIF(UNIX)

