/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2004
  Johan Skoglund
  skoglund@isy.liu.se
  CVL/isy
  university of Linkping
  sweden

BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

  //Warning this is a working version of the file not final API

  //measure the time for the different steps in the algorithm
  //#define use_timers

#ifdef __SSE__
#include <xmmintrin.h>
#endif
  
#include <Base/Image/ImageIO.hh>
#include "ColorHarris.hh"
#include "Gauss.hh"
#include "GaussDeriv.hh"
#include <Base/Image/ImageConvert.hh>

using namespace BIAS;
using namespace std;

#ifdef __SSE__
//needed because a=_mm_add_epi32(a,b) gives terrible code g++ 3.3.3-7 (redhat)
#define fastsum_epi32(sum,add)  \
asm("paddd %2,%0" \
    :"=x" (sum) \
    :"0" (sum),"x"(add))

//needed because a=_mm_add_epi16(a,b) gives terrible code g++ 3.3.3-7 (redhat)
#define fastsum_epi16(sum,add)  \
asm("paddw %2,%0" \
    :"=x" (sum) \
    :"0" (sum),"x"(add))

#endif

static int mytoplanar(const Image<unsigned char> &src,Image<unsigned char> &dst)
{
  //from imageconvert::ToPlanarrgb_
  BIASASSERT(src.IsInterleaved());
  BIASASSERT(src.GetChannelCount()==3);
  
  if(!dst.SamePixelAndChannelCount(src))
    {
      dst.Release();
      dst.Init(src.GetWidth(),src.GetHeight(),src.GetChannelCount(),ImageBase::ST_unsignedchar,false);
      dst.SetColorModel(ImageBase::CM_RGB);
    }

  int pixelcnt=src.GetPixelCount();

  unsigned char *red=dst.GetImageData();
  unsigned char *green=red+pixelcnt;
  unsigned char *blue=green+pixelcnt;
  unsigned char *imagedata=src.GetImageData();
  unsigned char *imagedatastop=imagedata+3*pixelcnt;

  while(imagedata<imagedatastop)
    {
      *red++=*imagedata++;
      *green++=*imagedata++;
      *blue++=*imagedata++;
    }
  return 0;
}

//special functions needed as long the the convolution interface is not fully specified
static int mysobel(const Image<unsigned char> &src,Image<signed char> &gradWE,
		   Image<signed char> &gradNS)
{
  BIASASSERT(src.IsInterleaved());
  unsigned char **im=src.GetImageDataArray();
  signed char **gx=gradWE.GetImageDataArray();
  signed char **gy=gradNS.GetImageDataArray();
  const int width=(int)src.GetWidth();
  const int height=(int)src.GetHeight();
  const int cc=src.GetChannelCount();

  unsigned minx, miny, maxx, maxy;
  src.GetROI(minx, miny, maxx, maxy);
  gradWE.SetROI(minx+1, miny+1, maxx-1, maxy-1);
  gradNS.SetROI(minx+1, miny+1, maxx-1, maxy-1);
  
#ifndef __SSE__
  for(int img_y=miny+1;img_y<maxy-1;img_y++)
    for(int img_x=minx+1;img_x<maxx-1;img_x++)
      {
	int sumx=0;
	int sumy=0;
	for(int cloop=0;cloop<cc;cloop++)
	  {
	    //sobelx=[1 0-1;2 0 -2;1 0 -1]
	    sumx=src.PixelValueInterleaved(img_x+1,img_y-1,cloop)-src.PixelValueInterleaved(img_x-1,img_y-1,cloop);
	    sumx+=2*(src.PixelValueInterleaved(img_x+1,img_y,cloop)-src.PixelValueInterleaved(img_x-1,img_y,cloop));
	    sumx+=src.PixelValueInterleaved(img_x+1,img_y+1,cloop)-src.PixelValueInterleaved(img_x-1,img_y+1,cloop);
	    gx[img_y][img_x*cc+cloop]=(sumx>>3);
	    
	    //sobely=[1 2 1;0 0 0;-1 -2 -1]

	    sumy=src.PixelValueInterleaved(img_x-1,img_y+1,cloop)+2*src.PixelValueInterleaved(img_x,img_y+1,cloop)
	      +src.PixelValueInterleaved(img_x+1,img_y+1,cloop);
	    sumy-=src.PixelValueInterleaved(img_x-1,img_y-1,cloop)+2*src.PixelValueInterleaved(img_x,img_y-1,cloop)
	      +src.PixelValueInterleaved(img_x+1,img_y-1,cloop);
	    gy[img_y][img_x*cc+cloop]=(sumy>>3);
	  }
      }
#endif
//   ImageIO::Write("gradnsold.mip",gradNS);
//   ImageIO::Write("gradweold.mip",gradWE);

#ifdef __SSE__
  for(int img_y=miny+1;img_y<maxy-1;img_y++)
    for(int img_x=minx+1;img_x<(maxx-1)*cc-16;img_x+=10)
//     for(int img_x=minx+1;img_x<maxx-1;img_x++)
      {
	//gradx
	unsigned char **srcida=src.GetImageDataArray();
	signed char **dstida=gradWE.GetImageDataArray();
	unsigned char *row1=srcida[img_y-1];
	unsigned char *row2=srcida[img_y];
	unsigned char *row3=srcida[img_y+1];

	__m128i inl1,inl2,inl3,zero;
	__m128i suml,sumh;
	__m128i up1,up2;

	zero=_mm_setzero_si128();
	suml=_mm_setzero_si128();
	sumh=_mm_setzero_si128();

	row1+=img_x-3;
	row2+=img_x-3;
	row3+=img_x-3;

	//load line 1-3
	inl1=_mm_loadu_si128((__m128i*)&row1[0]);
	inl2=_mm_loadu_si128((__m128i*)&row2[0]);
	inl3=_mm_loadu_si128((__m128i*)&row3[0]);

	//sub 1,1 
	up1=_mm_unpacklo_epi8(inl1,zero);
	up2=_mm_unpackhi_epi8(inl1,zero);
	suml=_mm_sub_epi16(suml,up1);
	sumh=_mm_sub_epi16(sumh,up2);

	//sub 1,2 *2
	up1=_mm_unpacklo_epi8(inl2,zero);
	up2=_mm_unpackhi_epi8(inl2,zero);
	up1=_mm_slli_epi16(up1,1);
	up2=_mm_slli_epi16(up2,1);
	suml=_mm_sub_epi16(suml,up1);
	sumh=_mm_sub_epi16(sumh,up2);

	//sub 1,3
	up1=_mm_unpacklo_epi8(inl3,zero);
	up2=_mm_unpackhi_epi8(inl3,zero);
	suml=_mm_sub_epi16(suml,up1);
	sumh=_mm_sub_epi16(sumh,up2);

	//rightshift 6 bytes
	inl1=_mm_srli_si128(inl1,6);
	inl2=_mm_srli_si128(inl2,6);
	inl3=_mm_srli_si128(inl3,6);

	//add 3,1
	up1=_mm_unpacklo_epi8(inl1,zero);
	up2=_mm_unpackhi_epi8(inl1,zero);
	suml=_mm_add_epi16(suml,up1);
	sumh=_mm_add_epi16(sumh,up2);

	//add 3,2 *2
	up1=_mm_unpacklo_epi8(inl2,zero);
	up2=_mm_unpackhi_epi8(inl2,zero);
	up1=_mm_slli_epi16(up1,1);
	up2=_mm_slli_epi16(up2,1);
	suml=_mm_add_epi16(suml,up1);
	sumh=_mm_add_epi16(sumh,up2);

	//add 3,3
	up1=_mm_unpacklo_epi8(inl3,zero);
	up2=_mm_unpackhi_epi8(inl3,zero);
	suml=_mm_add_epi16(suml,up1);
	sumh=_mm_add_epi16(sumh,up2);

	//>>3
	suml=_mm_srai_epi16(suml,3);
	sumh=_mm_srai_epi16(sumh,3);
	
	//store as short
	suml=_mm_packs_epi16(suml,sumh);
	_mm_storeu_si128((__m128i*)&gx[img_y][img_x],suml);

      }

 
  for(int img_y=miny+1;img_y<maxy-1;img_y++)
    for(int img_x=minx+1;img_x<(maxx-1)*cc-16;img_x+=10)
      {
	//grady

	unsigned char **srcida=src.GetImageDataArray();
	signed char **dstida=gradNS.GetImageDataArray();
	unsigned char *row1=srcida[img_y-1];
	unsigned char *row3=srcida[img_y+1];
	row1+=img_x-3;
	row3+=img_x-3;

	__m128i inl1,inl2,zero;
	__m128i suml,sumh;
	__m128i up1,up2;

	zero=_mm_setzero_si128();
	suml=_mm_setzero_si128();
	sumh=_mm_setzero_si128();
	inl1=_mm_loadu_si128((__m128i*)&row1[0]);
	inl2=_mm_loadu_si128((__m128i*)&row3[0]);
	
	//add 1,3
	up1=_mm_unpacklo_epi8(inl2,zero);
	up2=_mm_unpackhi_epi8(inl2,zero);
	suml=_mm_add_epi16(suml,up1);
	sumh=_mm_add_epi16(sumh,up2);

	//sub 1,1
	up1=_mm_unpacklo_epi8(inl1,zero);
	up2=_mm_unpackhi_epi8(inl1,zero);
	suml=_mm_sub_epi16(suml,up1);
	sumh=_mm_sub_epi16(sumh,up2);

	//right shift 3 bytes
	inl1=_mm_srli_si128(inl1,3);
	inl2=_mm_srli_si128(inl2,3);

	//add 2,3 *2
	up1=_mm_unpacklo_epi8(inl2,zero);
	up2=_mm_unpackhi_epi8(inl2,zero);
	up1=_mm_slli_epi16(up1,1);
	up2=_mm_slli_epi16(up2,1);
	suml=_mm_add_epi16(suml,up1);
	sumh=_mm_add_epi16(sumh,up2);

	//sub 2,3 *2
	up1=_mm_unpacklo_epi8(inl1,zero);
	up2=_mm_unpackhi_epi8(inl1,zero);
	up1=_mm_slli_epi16(up1,1);
	up2=_mm_slli_epi16(up2,1);
	suml=_mm_sub_epi16(suml,up1);
	sumh=_mm_sub_epi16(sumh,up2);

	//right shift 3 bytes
	inl1=_mm_srli_si128(inl1,3);
	inl2=_mm_srli_si128(inl2,3);

	//add 3,3
	up1=_mm_unpacklo_epi8(inl2,zero);
	up2=_mm_unpackhi_epi8(inl2,zero);
	suml=_mm_add_epi16(suml,up1);
	sumh=_mm_add_epi16(sumh,up2);

	//sub 3,1
	up1=_mm_unpacklo_epi8(inl1,zero);
	up2=_mm_unpackhi_epi8(inl1,zero);
	suml=_mm_sub_epi16(suml,up1);
	sumh=_mm_sub_epi16(sumh,up2);
	
	//>>3
	suml=_mm_srai_epi16(suml,3);
	sumh=_mm_srai_epi16(sumh,3);
	
	//save as short
	suml=_mm_packs_epi16(suml,sumh);
	_mm_storeu_si128((__m128i*)&gy[img_y][img_x],suml);
      }
#endif
//   ImageIO::Write("gradnsold.mip",gradNS);
//   ImageIO::Write("gradweold.mip",gradWE);

  return(0);
}

//special functions needed as long the the convolution interface is not fully specified
//this one handles planar images instead of interleaved
static int mysobel2(const Image<unsigned char> &src,Image<signed char> &gradWE,
		   Image<signed char> &gradNS)
{
  BIASASSERT(src.IsPlanar());
  
  unsigned char *im=src.GetImageData();
  signed char *gx=gradWE.GetImageData();
  signed char *gy=gradNS.GetImageData();
  int cc=src.GetChannelCount();
  int width=src.GetWidth();
  int height=src.GetHeight();
  
  unsigned minx, miny, maxx, maxy;
  src.GetROI(minx, miny, maxx, maxy);
  gradWE.SetROI(minx+1, miny+1, maxx-1, maxy-1);
  gradNS.SetROI(minx+1, miny+1, maxx-1, maxy-1);

  unsigned char *row1=im;
  unsigned char *row2=im+width;
  unsigned char *row3=row2+width;
  //first interesting point to calculate is 1,1
  gx+=width+1;
  gy+=width+1;

  //last interesting point width-1,height-1
  signed char *gxstop=gx+cc*width*height-2*width-2;
#ifndef __SSE__
  while(gx<=gxstop)
    {
      int sumx=0;
      int sumy=0;
      //sobelX=[1 0 -1;2 0 -2;1 0 -1]
      sumx-=row1[0]+2*row2[0]+row3[0];
      sumx+=row1[2]+2*row2[2]+row3[2];
      *gx++=sumx>>3;
      sumy-=row1[0]+2*row1[1]+row1[2];
      sumy+=row3[0]+2*row3[1]+row3[2];
      *gy++=sumy>>3;
      row1++;
      row2++;
      row3++;
    }
#else
  while(gx<=gxstop)
    {
      __m128i inl,zero;
      __m128i sumlx,sumhx,sumly,sumhy;
      __m128i up1,up2;

      zero=_mm_setzero_si128();
      sumlx=_mm_setzero_si128();
      sumhx=_mm_setzero_si128();
      sumly=_mm_setzero_si128();
      sumhy=_mm_setzero_si128();

      //row 1
      inl=_mm_loadu_si128((__m128i*)&row1[0]);
      up1=_mm_unpacklo_epi8(inl,zero);
      up2=_mm_unpackhi_epi8(inl,zero);
      //sub
      sumlx=_mm_sub_epi16(sumlx,up1);
      sumhx=_mm_sub_epi16(sumhx,up2);
      sumly=_mm_sub_epi16(sumly,up1);
      sumhy=_mm_sub_epi16(sumhy,up2);

      //row1[1] instead of row1[0]
      inl=_mm_srli_si128(inl,1);
      up1=_mm_unpacklo_epi8(inl,zero);
      up2=_mm_unpackhi_epi8(inl,zero);
      //sub *2
      up1=_mm_slli_epi16(up1,1);
      up2=_mm_slli_epi16(up2,1);
      sumly=_mm_sub_epi16(sumly,up1);
      sumhy=_mm_sub_epi16(sumhy,up2);

      //row1[2] instead of row1[1]
      inl=_mm_srli_si128(inl,1);
      up1=_mm_unpacklo_epi8(inl,zero);
      up2=_mm_unpackhi_epi8(inl,zero);
      //dx add dy sub
//       sumlx=_mm_add_epi16(sumlx,up1);
//       sumhx=_mm_add_epi16(sumhx,up2);
      fastsum_epi16(sumlx,up1);
      fastsum_epi16(sumhx,up2);

      sumly=_mm_sub_epi16(sumly,up1);
      sumhy=_mm_sub_epi16(sumhy,up2);

      //row 2
      inl=_mm_loadu_si128((__m128i*)row2);
      up1=_mm_unpacklo_epi8(inl,zero);
      up2=_mm_unpackhi_epi8(inl,zero);
      //sub*2
      up1=_mm_slli_epi16(up1,1);
      up2=_mm_slli_epi16(up2,1);
      sumlx=_mm_sub_epi16(sumlx,up1);
      sumhx=_mm_sub_epi16(sumhx,up2);

      //row2[2] instead of ror2[0]
      inl=_mm_srli_si128(inl,2);
      up1=_mm_unpacklo_epi8(inl,zero);
      up2=_mm_unpackhi_epi8(inl,zero);
      //dx add*2
      up1=_mm_slli_epi16(up1,1);
      up2=_mm_slli_epi16(up2,1);
//       sumlx=_mm_add_epi16(sumlx,up1);
//       sumhx=_mm_add_epi16(sumhx,up2);
      fastsum_epi16(sumlx,up1);
      fastsum_epi16(sumhx,up2);

      //row 3
      inl=_mm_loadu_si128((__m128i*)&row3[0]);
      up1=_mm_unpacklo_epi8(inl,zero);
      up2=_mm_unpackhi_epi8(inl,zero);
      //dx sub dy add
      sumlx=_mm_sub_epi16(sumlx,up1);
      sumhx=_mm_sub_epi16(sumhx,up2);
//       sumly=_mm_add_epi16(sumly,up1);
//       sumhy=_mm_add_epi16(sumhy,up2);
      fastsum_epi16(sumly,up1);
      fastsum_epi16(sumhy,up2);

      //row3[1] instead of row3[0]
      inl=_mm_srli_si128(inl,1);
      up1=_mm_unpacklo_epi8(inl,zero);
      up2=_mm_unpackhi_epi8(inl,zero);
      //dy add*2      
      up1=_mm_slli_epi16(up1,1);
      up2=_mm_slli_epi16(up2,1);
//       sumly=_mm_add_epi16(sumly,up1);
//       sumhy=_mm_add_epi16(sumhy,up2);
      fastsum_epi16(sumly,up1);
      fastsum_epi16(sumhy,up2);

      //row3[2] instead of row3[1]
      inl=_mm_srli_si128(inl,1);
      up1=_mm_unpacklo_epi8(inl,zero);
      up2=_mm_unpackhi_epi8(inl,zero);
      //add
//       sumlx=_mm_add_epi16(sumlx,up1);
//       sumhx=_mm_add_epi16(sumhx,up2);
//       sumly=_mm_add_epi16(sumly,up1);
//       sumhy=_mm_add_epi16(sumhy,up2);
      fastsum_epi16(sumlx,up1);
      fastsum_epi16(sumhx,up2);
      fastsum_epi16(sumly,up1);
      fastsum_epi16(sumhy,up2);

      //divide by 8
      sumlx=_mm_srai_epi16(sumlx,3);
      sumly=_mm_srai_epi16(sumly,3);
      sumhx=_mm_srai_epi16(sumhx,3);
      sumhy=_mm_srai_epi16(sumhy,3);

      //store as short
      sumlx=_mm_packs_epi16(sumlx,sumhx);
      sumly=_mm_packs_epi16(sumly,sumhy);

      _mm_storeu_si128((__m128i*)gx,sumlx);
      _mm_storeu_si128((__m128i*)gy,sumly);

      gx+=14;
      gy+=14;
      row1+=14;
      row2+=14;
      row3+=14;
    }

#endif
//   Image<signed char> foo;
//   ImageConvert::ToInterleaved(gradNS,foo);
//   ImageIO::Write("gradns.mip",foo);
//   ImageConvert::ToInterleaved(gradWE,foo);
//   ImageIO::Write("gradwe.mip",foo);

  return 0;
}

static void createst(Image<signed char> &dx,Image<signed char> &dy,Image<short> &st)
{
  int width=dx.GetWidth();
  int height=dx.GetHeight();

  if(!dx.SamePixelAndChannelCount(st))
    {
      st.Release();
      st.Init(width,height,3,ImageBase::ST_invalid,false);
      st.SetColorModel(ImageBase::CM_SymTensor2x2);
    }

  short **dxxp=st.GetImageDataArray();
  short **dxyp=dxxp+height;
  short **dyyp=dxyp+height;

  signed char **dxp=dx.GetImageDataArray();
  signed char **dyp=dy.GetImageDataArray();
  {
    signed char *dx=*dxp;
    signed char *dy=*dyp;
    short *dxx=*dxxp;
    short *dxy=*dxyp;
    short *dyy=*dyyp;
  for(unsigned int y=0;y<height*width;y++)
      {
	int txx,txy,tyy;
	short tx,ty;

#undef GRAD_UCHAR_ZERO
#define GRAD_UCHAR_ZERO 0
	tx=(*dx++)-GRAD_UCHAR_ZERO;
	ty=(*dy++)-GRAD_UCHAR_ZERO;

	txx=tx*tx;
	txy=tx*ty;
	tyy=ty*ty;

	tx=(*dx++)-GRAD_UCHAR_ZERO;
	ty=(*dy++)-GRAD_UCHAR_ZERO;

	txx+=tx*tx;
	txy+=tx*ty;
	tyy+=ty*ty;

	tx=(*dx++)-GRAD_UCHAR_ZERO;
	ty=(*dy++)-GRAD_UCHAR_ZERO;

	txx+=tx*tx;
	txy+=tx*ty;
	tyy+=ty*ty;

	*dxx++=txx/2;
	*dxy++=txy/2;
	*dyy++=tyy/2;
      }
  }
}

//from planar images
static void createst2(Image<signed char> &dx_plan,Image<signed char> &dy_plan,Image<short> &st)
{
  int width=dx_plan.GetWidth();
  int height=dx_plan.GetHeight();
  if(!dx_plan.SamePixelAndChannelCount(st))
    {
      st.Release();
      st.Init(width,height,3,ImageBase::ST_invalid,false);
      st.SetColorModel(ImageBase::CM_SymTensor2x2);
    }

  signed char *dxred=dx_plan.GetImageData();
  signed char *dxgreen=dxred+width*height;
  signed char *dxblue=dxgreen+width*height;
  signed char *dyred=dy_plan.GetImageData();
  signed char *dygreen=dyred+width*height;
  signed char *dyblue=dygreen+width*height;

  short *dxx=st.GetImageData();
  short *dxy=dxx+width*height;
  short *dyy=dxy+width*height;

#ifndef __SSE__
  for(int y=0;y<width*height;y++)
    {
      int txx,txy,tyy;
      int tx,ty;
      
      tx=*dxred++;
      ty=*dyred++;
      
      txx=tx*tx;
      txy=tx*ty;
      tyy=ty*ty;

      tx=*dxgreen++;
      ty=*dygreen++;
      
      txx+=tx*tx;
      txy+=tx*ty;
      tyy+=ty*ty;

      tx=*dxblue++;
      ty=*dyblue++;
      
      txx+=tx*tx;
      txy+=tx*ty;
      tyy+=ty*ty;

      //div by 2 because the summation of three colorchannels might give overflow otherwise
      *dxx++=txx/2;
      *dxy++=txy/2;
      *dyy++=tyy/2;
    }
#else
  for(int y=0;y<width*height;y+=8)
    {
      __m128i tx,ty,zero;
      __m128i sign;
      __m128i txshort;
      __m128i tyshort;
      __m128i txx1,txx2,txy1,txy2,tyy1,tyy2;
      __m128i mul1,mul2,mullow,mulhigh;

      txx1=_mm_setzero_si128();
      txx2=_mm_setzero_si128();
      txy1=_mm_setzero_si128();
      txy2=_mm_setzero_si128();
      tyy1=_mm_setzero_si128();
      tyy2=_mm_setzero_si128();
      zero=_mm_setzero_si128();

#define add_to_tensor \
      /*generate txshort,tyshort,extract tx0..7 ty0..7*/\
      sign=_mm_cmpgt_epi8(zero,tx);\
      txshort=_mm_unpacklo_epi8(tx,sign);\
      sign=_mm_cmpgt_epi8(zero,ty);\
      tyshort=_mm_unpacklo_epi8(ty,sign);\
      /*txx*/\
      mul1=_mm_mullo_epi16(txshort,txshort);\
      mul2=_mm_mulhi_epi16(txshort,txshort);\
      mullow=_mm_unpacklo_epi16(mul1,mul2);\
      mulhigh=_mm_unpackhi_epi16(mul1,mul2);\
      /*txx1=_mm_add_epi32(mullow,txx1);*/\
      /*txx2=_mm_add_epi32(mulhigh,txx2);*/\
      fastsum_epi32(txx1,mullow);fastsum_epi32(txx2,mulhigh);\
      /*txy*/\
      mul1=_mm_mullo_epi16(txshort,tyshort);\
      mul2=_mm_mulhi_epi16(txshort,tyshort);\
      mullow=_mm_unpacklo_epi16(mul1,mul2);\
      mulhigh=_mm_unpackhi_epi16(mul1,mul2);\
      /*txy1=_mm_add_epi32(mullow,txy1);*/\
      /*txy2=_mm_add_epi32(mulhigh,txy2);*/\
      fastsum_epi32(txy1,mullow);fastsum_epi32(txy2,mulhigh);\
      /*tyy*/\
      mul1=_mm_mullo_epi16(tyshort,tyshort);\
      mul2=_mm_mulhi_epi16(tyshort,tyshort);\
      mullow=_mm_unpacklo_epi16(mul1,mul2);\
      mulhigh=_mm_unpackhi_epi16(mul1,mul2);\
      /*tyy1=_mm_add_epi32(mullow,tyy1);*/\
      /*tyy2=_mm_add_epi32(mulhigh,tyy2);*/\
      fastsum_epi32(tyy1,mullow);fastsum_epi32(tyy2,mulhigh)

      tx=_mm_loadu_si128((__m128i*)dxred);
      ty=_mm_loadu_si128((__m128i*)dyred);
      add_to_tensor;
      tx=_mm_loadu_si128((__m128i*)dxgreen);
      ty=_mm_loadu_si128((__m128i*)dygreen);
      add_to_tensor;
      tx=_mm_loadu_si128((__m128i*)dxblue);
      ty=_mm_loadu_si128((__m128i*)dyblue);
      add_to_tensor;

      //divide by 2
      txx1=_mm_srai_epi32(txx1,1);
      txx2=_mm_srai_epi32(txx2,1);
      txy1=_mm_srai_epi32(txy1,1);
      txy2=_mm_srai_epi32(txy2,1);
      tyy1=_mm_srai_epi32(tyy1,1);
      tyy2=_mm_srai_epi32(tyy2,1);

      //store as short
      txx1=_mm_packs_epi32(txx1,txx2);
      txy1=_mm_packs_epi32(txy1,txy2);
      tyy1=_mm_packs_epi32(tyy1,tyy2);

      _mm_storeu_si128((__m128i*)dxx,txx1);
      _mm_storeu_si128((__m128i*)dxy,txy1);
      _mm_storeu_si128((__m128i*)dyy,tyy1);
      dxx+=8;
      dxy+=8;
      dyy+=8;
      dxred+=8;
      dyred+=8;
      dxgreen+=8;
      dygreen+=8;
      dxblue+=8;
      dyblue+=8;
    }
#endif
}

int ColorHarris::CreateSTColor(Image<unsigned char> &im,Image<short> &st,const float sigma1,const float sigma2)
{
  if(im.GetColorModel()!=ImageBase::CM_RGB) {
    BIASERR("create_ST_color only supports colormodel CM_RGB");
    return -1;
  }
//   Image<unsigned char> dx,dy;
#ifdef use_timers
  TimeMeasure time;
#endif
//   Image<signed char> dx,dy;
  static Image<signed char> dx_plan,dy_plan;
  unsigned int width,height;
  width=im.GetWidth();
  height=im.GetHeight();

#ifdef use_timers  
  time.Reset();
  time.start();
#endif
  static Image<unsigned char> im_planar;
  //I was not able to get ImageConvert::Toplanar to compile
  mytoplanar(im,im_planar);
#ifdef use_timers
  time.stop();
    cout <<"\nmytoplanar took:\n";
  time.print();
  time.Reset();
#endif

//   dx.Init(width,height,3);
//   dy.Init(width,height,3);
//   dx.SetColorModel(ImageBase::CM_RGB);
//   dy.SetColorModel(ImageBase::CM_RGB);

  if(!im.SamePixelAndChannelCount(dx_plan))
    {
      dx_plan.Release();
      dy_plan.Release();
      dx_plan.Init(width,height,3,ImageBase::ST_char,false);
      dy_plan.Init(width,height,3,ImageBase::ST_char,false);
      dx_plan.SetColorModel(ImageBase::CM_RGB);
      dy_plan.SetColorModel(ImageBase::CM_RGB);
    }
#ifdef use_timers
  time.start();
#endif  

  if(sigma1==0)
    {
//       mysobel(im,dx,dy);
      mysobel2(im_planar,dx_plan,dy_plan);
//       GradientSobel3x3<unsigned char,int> gr;
//       gr.Filter(im,dx,dy);
    }
  else
    {
      BIASERR("sigma1!=0 not implemented yet");
    }
#ifdef use_timers
  time.stop();
  cout << "\nsobel took:\n";
  time.print();

  time.Reset();
  time.start();
#endif

//   createst(dx,dy,st);
  createst2(dx_plan,dy_plan,st);

#ifdef use_timers
  time.stop();
  cout << "\nto tensor took:\n";
  time.print();

  time.Reset();
  time.start();
#endif

  if(sigma2!=0)
    {
//       Gauss<short,int> gauss;
      
//       BIASERR("sigma2!=0 not implemented yet");
      GaussDeriv<short> gauss;
      gauss.SetSigma(sigma2);
      gauss.SetOrder(0,0);
      gauss.Filter(st,st);
    }
  else
    cout <<"\nsigma2==0\n";

#ifdef use_timers
  time.stop();
  cout << "\ngauss took:\n";
  time.print();
#endif

  return(0);
}

int ColorHarris::HarrisValue(Image<short> &st,Image<int> &res)
{
  unsigned int width,height;
  width=st.GetWidth();
  height=st.GetHeight();

  if (st.GetWidth()!=res.GetWidth()||st.GetHeight()!=res.GetHeight()){
    if (!res.IsEmpty()){
      res.Release();
    }
    res.Init(width, height,1);
    res.SetColorModel(ImageBase::CM_Grey);
  }

  short *dxxp=st.GetImageData();
  short *dxyp=dxxp+height*width;
  short *dyyp=dxyp+height*width;
  int *resp=res.GetImageData();
  unsigned int x,y;

#ifndef __SSE__
  for(y=0;y<height;y++)
    for(x=0;x<width;x++)
      {
	int t11,t12,t22,temp;
	t11=*dxxp++;
	t12=*dxyp++;
	t22=*dyyp++;
	//det(t)-0.04*tr(t)^2
	//0.04*tr(t)^2~tr(t)^2 >> 5
	temp=(t11+t22)*(t11+t22);
	*resp++=t11*t22-t12*t12-(temp>>5);
      }
#else
  for(y=0;y<height*width;y+=8)
    {
      __m128i t11,t12,t22,t11_t22;
      __m128i mull,mulh,templ,temph;
      __m128i t11xt22l,t11xt22h;
      __m128i t12sql,t12sqh;
      
      t11=_mm_loadu_si128((__m128i*)dxxp);
      t12=_mm_loadu_si128((__m128i*)dxyp);
      t22=_mm_loadu_si128((__m128i*)dyyp);

      //calculate temp
      t11_t22=_mm_adds_epi16(t11,t22);
      mull=_mm_mullo_epi16(t11_t22,t11_t22);
      mulh=_mm_mulhi_epi16(t11_t22,t11_t22);
      templ=_mm_unpacklo_epi16(mull,mulh);
      temph=_mm_unpackhi_epi16(mull,mulh);

      //t11*t22
      mull=_mm_mullo_epi16(t11,t22);
      mulh=_mm_mulhi_epi16(t11,t22);
      t11xt22l=_mm_unpacklo_epi16(mull,mulh);
      t11xt22h=_mm_unpackhi_epi16(mull,mulh);

      //t12square
      mull=_mm_mullo_epi16(t12,t12);
      mulh=_mm_mulhi_epi16(t12,t12);
      t12sql=_mm_unpacklo_epi16(mull,mulh);
      t12sqh=_mm_unpackhi_epi16(mull,mulh);

      //temp>>5
      templ=_mm_srai_epi32(templ,5);
      temph=_mm_srai_epi32(temph,5);

      //get the result
      t11xt22l=_mm_sub_epi32(t11xt22l,t12sql);
      t11xt22h=_mm_sub_epi32(t11xt22h,t12sqh);
      t11xt22l=_mm_sub_epi32(t11xt22l,templ);
      t11xt22h=_mm_sub_epi32(t11xt22h,temph);
      
      //store result
      _mm_storeu_si128((__m128i*)resp    ,t11xt22l);
      _mm_storeu_si128((__m128i*)(resp+4),t11xt22h);
      resp+=8;
      dxxp+=8;
      dxyp+=8;
      dyyp+=8;
    }

#endif

  return 0;
}
