/*
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#ifndef __Mean_hh__
#define __Mean_hh__

#include "Convolution.hh"


namespace BIAS 
{
  /** @class Mean
      @ingroup g_filter
      @brief average mean filter
      
      @author woelk 09/2004 */
  template <class InputStorageType, class OutputStorageType>
  class Mean : public Convolution<InputStorageType, OutputStorageType>
  {
  public:
    Mean();

    Mean(Param& para, const std::string& postfix="");
    
    Mean(Param& para, const int group_id, const std::string& postfix);
    
    Mean(const Mean<InputStorageType, OutputStorageType>& m) {};
    
    ~Mean();
    
    /** generic rescale function
        support for ROI is still missing.
        @author woelk 09/2004 */
    virtual int Filter(const Image<InputStorageType>& src, 
                       Image<OutputStorageType>& dst);

    /** deprecated
        @author woelk 09/2004 */
    int FilterMean(const Image<InputStorageType>& src, 
                   Image<OutputStorageType>& dst);

    /** dst width = src width - 1, dst height = src height - 1
        @todo support for ROI is still missing.
        @author woelk  01/2003 */
    int FilterMean2x2(const Image<InputStorageType>& src, 
                      Image<OutputStorageType>& dst);

    /** dst must be initialized:
        dst width = src width - 1, dst height = src height - 1
        Does support ROI. No specialization for integer data types.
        @author woelk 09/2004 */
    int FilterMean2x2Grey(const Image<InputStorageType>& src, 
                          Image<OutputStorageType>& dst);

    /////////////////////////////////////////////////////////////////////////

    inline void SetSize(const int size) { *_MeanSize=size; }

    inline int GetSize() const { return *_MeanSize; }

    inline int *GetSizePointer(){ return _MeanSize;};

  protected:
    int *_MeanSize;

    void _AddParam(Param& para, const int group_id, 
                   const std::string& postfix);

    /** sets the vector members of Convolution
        @author woelk 09/2004 */
    void _SetKernel();

    using Convolution<InputStorageType, OutputStorageType>::_fm;
  }; // class




} // namespace

#endif // __Mean_hh__
