/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#ifndef __MEDIAN_HH__
#define __MEDIAN_HH__

#include <Base/Image/Image.hh> 
#include <FilterBase.hh>

namespace BIAS
{
  /** @class Median
      @ingroup g_filter
      @brief 2D median filter
    
      @todo reanimate this class using filter interface
  */ 
  template <class InputStorageType, class OutputStorageType> 
  class Median 
    : public FilterNToN<InputStorageType,  OutputStorageType>
  {
  public:
    Median();

    Median(Param& para, const std::string& postfix="");

    Median(Param& para, const int group_id, const std::string& postfix);

    virtual ~Median();

    /** @brief not yet implemented, should decide on the image types which
        FilterFunction should be called */
    virtual int Filter(const Image<InputStorageType>& src, 
                       Image<OutputStorageType>& dst);


    /** @brief 2D-Median filter, filtersize is 2*halfsize+1
        if minvalue is given, pixels with values smaller minvalue are ignored
        and target pixel is only set if half of all source-pixels in current
        region if greater minvalue.
        @author Jan-Friso Evers-Senne
        @date 2003-04-23    */
    
    int ComputeMedian(Image<InputStorageType>& src, 
                      Image<OutputStorageType>& dst,
                      unsigned int halfsize, float minvalue = FLT_MIN);
    
    template <class StorageType>
    int NonMaxSupression3x3(Image<InputStorageType>& src, 
                            Image<OutputStorageType>& dst);
    
    /** @brief 2D-Median filter, filtersize is 2*halfsize+1
        Zero values are ignored
        src and dst cannot be identical
        also removes single values in detection window
        @author woelk 09 2003 */
    int FilterIgnoreZero(Image<InputStorageType>& src, 
                         Image<OutputStorageType>& dst,
                         unsigned int halfsize);

    template <class StorageType>
    int FilterIgnoreZero5x5(Image<InputStorageType>& src, 
                            Image<OutputStorageType>& dst);

    /** fast median on double array, faster than sort 
        @author woelk 01/2004 */
    InputStorageType Filter(InputStorageType arr[], int n);

  };




}// namepspace 

#endif
