/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include "../HMatrix.hh"

using namespace std;


// ---------------------------------------------------------------------------
int main(int argc, char *argv[]) {
    
  BIAS::HMatrix TheoH,EstH;

  // set homography TheoH
  TheoH.set_identity(); TheoH[0][2]=10; TheoH[1][2]=10;
    
  // H-matrix
  std::vector<BIAS::HomgPoint2D> points_src;
  std::vector<BIAS::HomgPoint2D> points_dest, points_dest_with_out;
  
  // build pointsets 2D for computation
  int no_outlier=0;
  for (int i = 0; i <100; i++){ 
    // 2D point
    BIAS::HomgPoint2D point2d_im1;
    for (int k=0; k<2; k++)
      // point coordinate between 0 and 1000
      point2d_im1[k]=10.0*rand()/(RAND_MAX+1.0);
    point2d_im1[2]=1;
    // project the point2d to the second image
    BIAS::HomgPoint2D point2d_im2 = TheoH*point2d_im1;
    point2d_im2.Homogenize();
    
    // build correspondences
    points_src.push_back(point2d_im1);
    points_dest.push_back(point2d_im2);
    
    if ((rand()/(RAND_MAX+1.0)) > 0.8){
      no_outlier++;
      BIAS::HomgPoint2D point2d_err(0.0,0.0);
      points_dest_with_out.push_back(point2d_err);
    } else {
      points_dest_with_out.push_back(point2d_im2);
    }

  }

  cout << "Number of outliers is " << no_outlier << endl;

  
  // estimation of H with MLESAC
  EstH.ComputeMLESAC(points_src,points_dest_with_out,1.0);


  // write estimated H to file;
  string resultNameH="H_MLESAC.txt";
  ofstream fs_estH;
  fs_estH.open(resultNameH.c_str() );
  fs_estH << EstH/EstH[2][2];
  fs_estH.close();
  
  
  cout << " ### "<<argv[0]<< " DONE ###" << endl;
  return 0;
}

