/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include "Quadric3D.hh"

using namespace BIAS;
using namespace std;

Quadric3D::Quadric3D(const HomgPoint3D& C, 
                     const CovMatrix3x3& cov, 
                     const double &dScale) {
  BIAS::Matrix3x3<double> InvCov = cov * (dScale * dScale);
  InvCov = InvCov.Invert();

  BIAS::Matrix4x4<double> EllipsoidAtOrigin;
  EllipsoidAtOrigin.SetIdentity();
  for (unsigned int i=0; i<3; i++) {
    for (unsigned int j=0; j<3; j++) {
      EllipsoidAtOrigin[i][j] =  InvCov[i][j];
    }
  }
  EllipsoidAtOrigin[3][3] = -1;

  Matrix4x4<double> MoveToOrigin, MoveToOriginT;
  MoveToOrigin.SetIdentity();
  
  for (int i=0; i<3; i++) {
    MoveToOrigin[i][3]  =  -C[i];
  }
  MoveToOrigin[3][3] = C[3];
  MoveToOriginT = MoveToOrigin.Transpose();
 
  (*this) = MoveToOriginT * (EllipsoidAtOrigin * MoveToOrigin);
  // Quadric has radii dScale*sigma_i and is centered at point3d now

  Normalize();
}

#define SVD_ZERO_THRESH DBL_EPSILON * 16.0
unsigned int Quadric3D::GetSignature() {
  // do singular value decomposition of this
  SVD svd(*this);
  Vector4<double> S;
  Matrix4x4<double> VT;
  S = BIAS::Vector4<COVMATRIX3X3_TYPE>(svd.GetS());
  VT = svd.GetVT();
  
  // now we need the eigenvalues (with sign !)
  // compute this by projecting the (normalized) eigenvectors onto their images
  Vector4<double> eigenvector, eigenimage;
  signed char Signature = 0;
  for (unsigned int i=0; i<4; i++) {
    // get ith eigenvector and its image
    eigenvector = VT.GetRow(i);
    eigenimage = (*this)*eigenvector; 
    // compute scalar product, we are not "thinking" homogenized now
    // this is simply an analysis of a 4x4 matrix !
    S[i] = eigenimage[0] * eigenvector[0] + eigenimage[1] * eigenvector[1]+
      eigenimage[2] * eigenvector[2]+  eigenimage[3] * eigenvector[3];
    if (S[i]<-SVD_ZERO_THRESH) Signature--;
    else if (S[i]>SVD_ZERO_THRESH) Signature++;
  }
    
  // return number of eigenvalues with one sign minus number of eigenvalues
  // with opposite sign, choosing signs such that difference is not negative
  if (Signature<0) return (unsigned int)(-Signature);
  return (unsigned int)Signature;
}
