/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include "GuiEnhanced.hh"

#include <unistd.h>
#include <Base/Image/Bresenham.hh>

using namespace BIAS;
using namespace std;

GuiEnhanced::GuiEnhanced()
  : GuiGTK()
{}


GuiEnhanced::~GuiEnhanced()
{}

void GuiEnhanced::GetZoomImage(unsigned int xpos, unsigned int ypos, 
                               unsigned int factor, unsigned int size,
                               Image<unsigned char>& zoomimage)
{
  if (!zoomimage.IsEmpty())
    zoomimage.Release();
  zoomimage.Init(2*factor*size+1, 2*factor*size+1, nChannels_);
  
  register unsigned char **ida = zoomimage.GetImageDataArray();
  register unsigned int xstart, ystart;
  register unsigned char *source=new unsigned char[nChannels_];
  
  for (register int x=(int)xpos-(int)size; x<(int)xpos+(int)size; x++){
    for (register int y=(int)ypos-(int)size; y<(int)ypos+(int)size; y++){
      if (x<0 || y<0 || x>=(int)Width_ || y>=(int)Height_)
        for (unsigned int i=0; i< nChannels_; i++)
          source[i] = 0;
      else
        for (unsigned int i=0; i< nChannels_; i++)
          source[i] = Buffer_[Width_*y*nChannels_+x*nChannels_+i];
      xstart = (x-xpos+size)*factor;
      ystart = (y-ypos+size)*factor;
      for (register int i=0; i<(int)factor; i++){
        for (register int j=0; j<(int)factor; j++){
          for (unsigned int c=0; c<nChannels_; c++){
            ida[ystart+i][xstart*nChannels_+j*nChannels_+c] = source[c];
          }
        }
      }
    }
  }
  delete[] source;
}

void GuiEnhanced::GetZoomImage(unsigned int x, unsigned y, unsigned factor, 
                               unsigned size, Image<unsigned char>& zoomimage,
                               int& xoffset, int& yoffset)
{
  GetZoomImage(x, y, factor, size, zoomimage);
  if (IsPyramid_==0 || (x<Pyramid0Width_+PYRAMID_BORDER+(PYRAMID_BORDER>>1))){
    xoffset=x-size;
    yoffset=y-size;
    //cerr << "no pyramid: xoffset= "<<xoffset<<"\tyoffset= "<<yoffset<<endl;
  } else {
    xoffset = x-size-Pyramid0Width_-2*PYRAMID_BORDER;
    unsigned height = Pyramid0Height_>>1;
    int index = 1;
    while (y>height) {
      y=y-height-PYRAMID_BORDER;
      height=height<<1;
      index++;
    }
    //cerr << "index: "<<index<<"\txoffset= "<<xoffset<<"\tyoffset= "<<yoffset<<endl;
  }
}

void GuiEnhanced::GetZoomImage(unsigned int xpos, unsigned int ypos, 
                               unsigned int factor, unsigned int size,
                               ImageBase& zoomimage)
{
  if (!zoomimage.IsEmpty())
    zoomimage.Release();
  zoomimage.Init(2*factor*size+1, 2*factor*size+1, 
                 OrigPicture_.GetChannelCount(), 
                 OrigPicture_.GetStorageType());
  register unsigned int xstart, ystart;
  unsigned stsize=OrigPicture_.GetDepth(), msize=stsize*OrigPicture_.GetChannelCount();
 
  register char **ida = (char **)zoomimage.GetImageDataArray();
  register char *source=new char[size];
  
  for (register int x=(int)xpos-(int)size; x<(int)xpos+(int)size; x++){
    for (register int y=(int)ypos-(int)size; y<(int)ypos+(int)size; y++){
      if (x<0 || y<0 || x>=(int)Width_ || y>=(int)Height_)
        for (unsigned int i=0; i< msize; i++)
          source[i] = 0;
      else
        for (unsigned int i=0; i< msize; i++)
          source[i] = ((char *)(OrigPicture_.GetImageData()))[Width_*y*msize+x*msize+i];
      xstart = (x-xpos+size)*factor;
      ystart = (y-ypos+size)*factor;
      for (register int i=0; i<(int)factor; i++){
        for (register int j=0; j<(int)factor; j++){
          for (unsigned int c=0; c<msize; c++){
            ida[ystart+i][xstart*msize+j*msize+c] = source[c];
          }
        }
      }
    }
  }
  delete[] source;
}

void GuiEnhanced::GetZoomImage(unsigned int x, unsigned y, unsigned factor, 
                               unsigned size, ImageBase& zoomimage,
                               int& xoffset, int& yoffset)
{
  GetZoomImage(x, y, factor, size, zoomimage);
  if (IsPyramid_==0 || (x<Pyramid0Width_+PYRAMID_BORDER+(PYRAMID_BORDER>>1))){
    xoffset=x-size;
    yoffset=y-size;
    //    cerr << "no pyramid: xoffset= "<<xoffset<<"\tyoffset= "<<yoffset<<endl;
  } else {
    xoffset = x-size-Pyramid0Width_-2*PYRAMID_BORDER;
    unsigned height = Pyramid0Height_>>1;
    int index = 1;
    while (y>height) {
      y=y-height-PYRAMID_BORDER;
      height=height<<1;
      index++;
    }
    // cerr << "index: "<<index<<"\txoffset= "<<xoffset<<"\tyoffset= "<<yoffset<<endl;
  }
}

int GuiEnhanced::SelectPoint(int point[2])
{
  int result = 0;
  GuiGTK zoomgui;
  Image<unsigned char> zoomimage;
  bool pressed=false;
  int button=-1, x, y, xz, yz;
  unsigned int factor = 15, size = 15;
  int tmpx, tmpy;

  while (pressed !=true)
    WaitForMouseButtonEvent(pressed, button, x, y);
  if (button==1){
    GetZoomImage(x, y, factor, size, zoomimage);
    zoomgui.SetFactorOffset(x-size, y-size, factor);
    zoomgui.ShowImage(zoomimage);
    button = -1;
    pressed = false;
    while (pressed!=true)
      zoomgui.WaitForMouseButtonEvent(pressed, button, xz, yz);
    if (button==1){
      tmpx = (int)x - (int)size + xz / (int)factor;
      tmpy = (int)y - (int)size + yz / (int)factor;
      if (tmpx < 0 || tmpx >= (int)Width_ || tmpy < 0 || tmpy >= (int)Height_){
        point[0] = point[1] = 0;
        BIASERR("invalid selection");
        result = -1;
      } else {
        point[0] = tmpx;
        point[1] = tmpy;
        result = 0;
      }
    } else { // not left mouse button, canceled selection
      result=-1;
    }
    zoomgui.DestroyWindow();
  } else { // not left mouse button, canceled selection
    result=-1;
  }

  return result;
}


int GuiEnhanced::SelectRectangle(int upperleft[2], 
                                 int lowerright[2])
{
  int result = 0;
  bool pressed=false;
  int button=-1, lastx=0, lasty=0;
  bool again = true;
  int size = Width_ * Height_ * nChannels_;
  guchar *OrigBuffer = new guchar[size];
  unsigned int tl[2], br[2];

  gdk_threads_enter();
  memcpy(OrigBuffer, Buffer_, size);
  gdk_threads_leave();

  while (again){
    //     cerr << "start" << endl;
    again=false;
    int x, y;
    while (pressed != true || button != 1){
      pressed=false;
      button=-1;
      
      WaitForMouseButtonEvent(pressed, button, x, 
                              y);
//       WaitForMouseButtonEvent(pressed, button, (int)upperleft[0], 
//                               (int)upperleft[1]);
      upperleft[0]=x;
      upperleft[1]=y;
    }
    //cerr << "first point: (" << x <<", "<<y<<")\n";
 //     cerr << "first point: (" << (int)upperleft[0] << ", " 
//          << (int)upperleft[1] <<")"<< endl;

    pressed=false;
    button=-1;
    while (CheckForMouseButtonEvent(pressed, button, lowerright[0], 
                                    lowerright[1]) || pressed != true){
      if (lastx != MousePosX_ || lasty != MousePosY_){
        gdk_threads_enter();
        memcpy(Buffer_, OrigBuffer, size);
        gdk_threads_leave();
        if ((int)upperleft[0]<MousePosX_){
          tl[0] = upperleft[0];
          br[0] = MousePosX_;
        } else {
          tl[0] = MousePosX_;
          br[0] = upperleft[0];
        }
        if ((int)upperleft[1]<MousePosY_){
          tl[1] = upperleft[1];
          br[1] = MousePosY_;
        } else {
          tl[1] = MousePosY_;
          br[1] = upperleft[1];
        }
        
        DrawRectangle(tl, br);
        lastx = MousePosX_;
        lasty = MousePosY_;
      }
      usleep(1000);
    }
    //    cerr << "button: " << button << endl;
    if (button != 1){
      //cerr << "again" << endl;
      again=true;
    }
    gdk_threads_enter();
    memcpy(Buffer_, OrigBuffer, size);
    Redraw_();
    gdk_threads_leave();
  }
  upperleft[0]=tl[0];
  upperleft[1]=tl[1];
  lowerright[0]=br[0];
  lowerright[1]=br[1];
  return result;
}


int GuiEnhanced::SelectQuadrate(int upperleft[2], 
                                int lowerright[2])
{

  int result = 0;
  bool pressed=false;
  int button=-1, lastx=0, lasty=0;
  bool again = true;
  int size = Width_ * Height_ * nChannels_;
  guchar *OrigBuffer = new guchar[size];
  unsigned int tl[2], br[2];

  gdk_threads_enter();
  memcpy(OrigBuffer, Buffer_, size);
  gdk_threads_leave();

  while (again){
    //     cerr << "start" << endl;
    again=false;
    
    while (pressed != true || button != 1){
      pressed=false;
      button=-1;
      WaitForMouseButtonEvent(pressed, button, upperleft[0], 
                              upperleft[1]);
    }

    //     cerr << "first point: (" << (int)upperleft[0] << ", " 
    // 	 << (int)upperleft[1] <<")"<< endl;

    pressed=false;
    button=-1;
    while (CheckForMouseButtonEvent(pressed, button, lowerright[0], 
                                    lowerright[1]) || pressed != true){
      if (lastx != MousePosX_ || lasty != MousePosY_){
        gdk_threads_enter();
        memcpy(Buffer_, OrigBuffer, size);
        gdk_threads_leave();

        int mp_x = MousePosX_;
        int mp_y = MousePosY_;

        int rect_width = abs((int) upperleft[0] - mp_x);
        int rect_height= abs((int) upperleft[1] - mp_y);

        // In every quadrant there are two possibilitiest for the 
        // mouse point: Above or under the diagonal
	
        // The mouse point is in ...
        if ((int) upperleft[0] < mp_x) {
          if ((int) upperleft[1] < mp_y) {

            // ... quadrant IV
            tl[0] = upperleft[0];
            tl[1] = upperleft[1];
            if (rect_width < rect_height) {
              mp_y = (int) upperleft[1] + rect_width;
            } else {
              mp_x = (int) upperleft[0] + rect_height;
            }
            br[0] = mp_x;
            br[1] = mp_y;
          } else {

            // ... quadrant I
            tl[0] = upperleft[0];	    
            br[1] = upperleft[1];
            if (rect_width < rect_height) {
              mp_y = (int) upperleft[1] - rect_width;
            } else {
              mp_x = (int) upperleft[0] + rect_height;
            }
            tl[1] = mp_y;
            br[0] = mp_x;
          }
        } else {
          if ((int) upperleft[1] < mp_y) {

            // ... quadrant III
            tl[1] = upperleft[1];	    
            br[0] = upperleft[0];
            if (rect_width < rect_height) {
              mp_y = (int) upperleft[1] + rect_width;
            } else {
              mp_x = (int) upperleft[0] - rect_height;
            }
            tl[0] = mp_x;
            br[1] = mp_y;
          } else {

            // ... quadrant II
            br[0] = upperleft[0];
            br[1] = upperleft[1];
            if (rect_width < rect_height) {
              mp_y = (int) upperleft[1] - rect_width;
            } else {
              mp_x = (int) upperleft[0] - rect_height;
            }
            tl[0] = mp_x;
            tl[1] = mp_y;
          }
        }

        DrawRectangle(tl,br);
        lastx = MousePosX_;
        lasty = MousePosY_;
      }
      usleep(1000);
    }
    //    cerr << "button: " << button << endl;
    if (button != 1){
      //cerr << "again" << endl;
      again=true;
    }
    gdk_threads_enter();
    memcpy(Buffer_, OrigBuffer, size);
    Redraw_();
    gdk_threads_leave();
  }
  upperleft[0]=tl[0];
  upperleft[1]=tl[1];
  lowerright[0]=br[0];
  lowerright[1]=br[1];
  return result;
} // SelectQuadrate

int GuiEnhanced::SelectLine(int start[2], int end[2])
{
  int result=0;
  int size = Width_ * Height_ * nChannels_;
  guchar *OrigBuffer = new guchar[size];
  bool pressed=false;
  int button=-1;
  int last[2];

  gdk_threads_enter();
  memcpy(OrigBuffer, Buffer_, size);
  gdk_threads_leave();

  while (!pressed)
    WaitForMouseButtonEvent(pressed, button, start[0], 
			    start[1]);

  if (button==1){
    last[0]=start[0];
    last[1]=start[1];
    pressed=false;
    button=-1;
    while (!CheckForMouseButtonEvent(pressed, button, end[0], 
				     end[1]) || pressed != true){
      if ((int)last[0] != MousePosX_ || (int)last[1] != MousePosY_){
	gdk_threads_enter();
	memcpy(Buffer_, OrigBuffer, size);
	gdk_threads_leave();
	last[0] = MousePosX_;
	last[1] = MousePosY_;
	DrawLine(start,last);
	gdk_threads_enter();
	Redraw_();
	gdk_threads_leave();
      }
      usleep(1000);
    }
    if (button!=1) {// wrong button while selecting end
      result=-1;
    }
  } else { // wrong button while selecting start
    result=-1;
  }
  gdk_threads_enter();
  memcpy(Buffer_, OrigBuffer, size);
  Redraw_();
  gdk_threads_leave();
  return result;
}


int GuiEnhanced::SelectLineZoom(int start[2],int end[2])
{
  int result=0;
  int size = Width_ * Height_ * nChannels_;
  guchar *OrigBuffer = new guchar[size];
  bool pressed=false;
  int button=-1, xz, yz;;
  int last[2];
  GuiEnhanced zoomgui;
  Image<unsigned char> zoomimage;
  unsigned int factor=15, zoomsize=15;
  int tmpx, tmpy;

  gdk_threads_enter();
  memcpy(OrigBuffer, Buffer_, size);
  gdk_threads_leave();
  if (SelectPoint(start)==0){
    last[0]=start[0];
    last[1]=start[1];
    pressed=false;
    button=-1;
    while (!CheckForMouseButtonEvent(pressed, button, end[0], 
				     end[1]) || pressed != true){
      if ((int)last[0] != MousePosX_ || (int)last[1] != MousePosY_){
	gdk_threads_enter();
	memcpy(Buffer_, OrigBuffer, size);
	gdk_threads_leave();
	last[0] = MousePosX_;
	last[1] = MousePosY_;

	DrawLine(start, last);
	gdk_threads_enter();
	Redraw_();
	gdk_threads_leave();
      }
      usleep(1000);
    }
    if (button==1) {
      gdk_threads_enter();
      memcpy(Buffer_, OrigBuffer, size);
      Redraw_();
      gdk_threads_leave();
      GetZoomImage(end[0], end[1], factor, zoomsize, zoomimage);
      zoomgui.SetFactorOffset(end[0]-zoomsize, end[1]-zoomsize, factor);
      zoomgui.ShowImage(zoomimage);
      button = -1;
      pressed = false;
      while (pressed!=true)
	zoomgui.WaitForMouseButtonEvent(pressed, button, xz, yz);
      if (button==1){
	tmpx = (int)end[0] - (int)zoomsize + xz / (int)factor;
	tmpy = (int)end[1] - (int)zoomsize + yz / (int)factor;
	//cerr << "end: " << end[0] << " " << end[1] << "  xz " << xz 
	//<< " yz: " << yz << "  tmpx " << tmpx << "  tmpy " << tmpy << endl;
	if (tmpx < 0 || tmpx >= (int)Width_ || tmpy < 0 || tmpy >= (int)Height_){
	  end[0] = end[1] = 0;
	  BIASERR("invalid selection");
	  result = -1;
	} else {
	  end[0] = tmpx;
	  end[1] = tmpy;
	}
      } else { // not left mouse button, canceled selection
	result=-1;
      }
      zoomgui.DestroyWindow();
    } else { // wrong button while selecting end
      result=-1;
    }
  } else { // wrong button while selecting start
    result=-1;
  }
  gdk_threads_enter();
  memcpy(Buffer_, OrigBuffer, size);
  Redraw_();
  gdk_threads_leave();
  return result;
}

void GuiEnhanced::DrawRectangle(unsigned int tl[2], unsigned int br[2])
{
  //cerr <<"draw rectangle ("<<tl[0]<<", "<<tl[1]<<") ("<<br[0]<<" ,"<<br[1]<<")   "<<nChannels_<<"\n";

  switch(nChannels_){
  case 1:
    // top
    gdk_threads_enter();
    for (int i=0; i< (int)(br[0]-tl[0]); i++)
      Buffer_[tl[1]*Width_+tl[0]+i]=
        (Buffer_[tl[1]*Width_+tl[0]+i]>128)?((gchar)0):((gchar)255);
    // bottom
    for (int i=0; i< (int)(br[0]-tl[0]); i++)
      Buffer_[br[1]*Width_+tl[0]+i]=
        (Buffer_[br[1]*Width_+tl[0]+i]>128)?((gchar)0):((gchar)255);
    // left and right
    for (int i=0; i < (int)(br[1]-tl[1]); i++){
      Buffer_[(tl[1]+i)*Width_+tl[0]]=
        (Buffer_[(tl[1]+i)*Width_+tl[0]]>128)?((gchar)0):((gchar)255);
      Buffer_[(tl[1]+i)*Width_+br[0]]=
        (Buffer_[(tl[1]+i)*Width_+br[0]]>128)?((gchar)0):((gchar)255);
    }
    gdk_threads_leave();
    break;
  case 3:
    gdk_threads_enter();
    for (int i=0; i< (int)(br[0]-tl[0]); i++){
      for (int j=0; j<3; j++){
        Buffer_[tl[1]*Width_*3+tl[0]*3+i*3+j]=
          (Buffer_[tl[1]*Width_*3+tl[0]*3+i*3+j]>128)?((gchar)0):((gchar)255);
        Buffer_[br[1]*Width_*3+tl[0]*3+i*3+j]=
          (Buffer_[br[1]*Width_*3+tl[0]*3+i*3+j]>128)?((gchar)0):((gchar)255);
      }
    }
    // left and right
    for (int i=0; i< (int)(br[1]-tl[1]); i++){
      for (int j=0; j<3; j++){
        Buffer_[(tl[1]+i)*Width_*3+tl[0]*3+j]=
          (Buffer_[(tl[1]+i)*Width_*3+tl[0]*3+j]>128)?((gchar)0):((gchar)255);
        Buffer_[(tl[1]+i)*Width_*3+br[0]*3+j]=
          (Buffer_[(tl[1]+i)*Width_*3+br[0]*3+j]>128)?((gchar)0):((gchar)255);
      }
    }
    gdk_threads_leave();
    break;
  default:
    BIASERR("invalid channel count " << nChannels_);
    break;
  }
  gdk_threads_enter();
  Redraw_();
  gdk_threads_leave();
}


void GuiEnhanced::DrawLine(int start[2], int end[2])
{
  int next[2];
  int adr;
  Bresenham bres(start, end);
  while (bres.GetNext(next)){
    adr=next[1]*Width_*nChannels_+next[0]*nChannels_;
    for (register unsigned int c=0; c<nChannels_ ; c++)
      Buffer_[adr+c]=(Buffer_[adr+c]>128)?((gchar)0):((gchar)255);
  }
  gdk_threads_enter();
  Redraw_();
  gdk_threads_leave();
}
