/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef  WIN32
  //no threads in WIN32
  #include <unistd.h>
  #include <pthread.h>
#endif //WIN32

#include <sstream>
#include "GuiWX.hh"
#include <Base/Image/ImageIO.hh>

using namespace BIAS;
using namespace std;



bool GuiWX::ThreadInitialized_ = false;
bool GuiWX::MainThreadRunning_ = false;
bool GuiWX::WxInitialized = false;
    
wxApp * GuiWX::AppApp_ = NULL;

unsigned short int GuiWX::WindowCounter_ = 0;


wxMutex GuiWX::GuiWxMutex_;

/// default constructor
WxMainThread::WxMainThread(){}


/// default constructor
WxMainThread::WxMainThread(GuiWX *gui)
{
  MyGui_ = gui;
}


/// default destructor
WxMainThread::~WxMainThread()
{
}


/** Each from wxThread derived class needs the method ::Entry()
    which is called when a thread is run.
*/
void *WxMainThread::Entry()
{
  // hi-jack wxTheApp and start the mainloop
   MyGui_->AppApp_ ->MainLoop();
//   while (MyGui_->AppApp_->Pending())
//     MyGui_->AppApp_->Dispatch();

  delete MyGui_->AppApp_;
  MyGui_->AppApp_ = NULL;
  return NULL;
}


/** Just like Entry, every wxThread needs the method ::OnExit()
    which is called when a thread terminates or is terminated.
    It won't be executed, when a thread is killed.
*/
void WxMainThread::OnExit()
{
  MyGui_->MainThreadRunning_ = false;
}


GuiWX::GuiWX(bool ShowMousePos) : GuiBase(ShowMousePos)
{  
  TmpWxImg_ = NULL;
  Frame_ = NULL;
  Init_(ShowMousePos);
  Title_="no title";
}


GuiWX::GuiWX(const string& Title,bool ShowMousePos) : GuiBase(ShowMousePos)
{   
  TmpWxImg_ = NULL;
  Frame_ = NULL;
  Init_(ShowMousePos);
  Title_=Title;
}

GuiWX::GuiWX(const string& Title) : GuiBase(true)
{   
  TmpWxImg_ = NULL;
  Frame_ = NULL;
  Init_(true);
  Title_=Title;
}


extern int wxEntryStart(int &argc,  char *argv[]);
extern int wxEntryInitGui();

void GuiWX::Init_(bool ShowMousePos)
{
  GuiBase::Init_(ShowMousePos);

  int argc=1;
  char *argv[1] ={"blafasel"};

  Buffer_ = NULL;  
  if (!WxInitialized) {
    AppApp_ = new wxApp;
    // set the global wxTheApp pointer to satisfy wxWindows
    wxTheApp = AppApp_;
    if (::wxEntryStart(argc, argv)) {
      BIASERR("wxEntryStart() failed");
      return ;
    }
    wxTheApp->argc = argc;
    wxTheApp->argv = argv;
    wxTheApp->SetAppName(Title_.c_str());
    int retValue;
    retValue = wxEntryInitGui();
    WxInitialized = true;
  }
}


GuiWX::~GuiWX()
{
}


void GuiWX::SetTitle(string const& Title)
{
  wxString tmp(Title.c_str());

  if ( WindowCreated_ )
    Frame_->SetTitle( tmp );

  Title_=Title;
  return;
}


int GuiWX::DestroyWindow()
{   
  if (WindowCreated_){
    Frame_->Hide();
    WindowCreated_=false;
    BIASDOUT(D_GUI_MUTEX,"DestroyWindow(): waiting for mutex..."); 
    GuiWxMutex_.Lock();
    BIASDOUT(D_GUI_MUTEX,"DestroyWindow(): ...got mutex");
    WindowCounter_--;

//     if (WindowCounter_ == 0){
//       delete WxMainLoop_;
//       WxMainLoop_ = NULL;
//     }
    GuiWxMutex_.Unlock();
//     delete Frame_;
//     Frame_ = NULL;
  }
  return 0;
}


int GuiWX::CreateWindow_(Image<unsigned char> &image)
{
  if (Frame_ == NULL)
    Frame_ = new TopFrame(this, image);
  Frame_->SetTitle(Title_.c_str());
  Frame_->Show(TRUE);

  BIASDOUT(D_GUI_MUTEX,"CreateWindow: Waiting for mutex" );
  GuiWxMutex_.Lock();
  BIASDOUT(D_GUI_MUTEX,"CreateWindow: Got mutex")
  if ( ! MainThreadRunning_ )
    {  
      MainThreadRunning_ = true;
      WxMainLoop_ = new WxMainThread(this);
      WxMainLoop_->Create() ;
      WxMainLoop_->Run();
    }
  WindowCounter_++;
  GuiWxMutex_.Unlock();
  WindowCreated_=true;

  return 0;
}

#include <Base/Image/ImageIO.hh>


int GuiWX::ShowConvertedImage_(Image<unsigned char> &image)
{

  // Create a window if not already done, resize it if necessary
  // and display the already converted image
  if ( !WindowCreated_ ) 
    CreateWindow_(image);
  Frame_->Canvas_->Show(image,"foo");
  return 0;
}





// =============  EVENTS FOR TOP_FRAME ===============================
BEGIN_EVENT_TABLE(TopFrame, wxFrame)
  EVT_CHAR( TopFrame::OnChar )
  EVT_CLOSE( TopFrame::OnClose )
END_EVENT_TABLE()


// =============  METHODS FOR TOP_FRAME ==============================
TopFrame::TopFrame( GuiWX *gui, Image<unsigned char> &image )
  : wxFrame( (wxFrame*)NULL, -1, "No Title", wxPoint(-1,-1), wxSize(800,600))
{

  CreateStatusBar(3);
  int widths[] = { -1, 150, -1 }; // in array, -1 means variable width
  SetStatusWidths( 3, widths );
  MyGui_ = gui;
  Canvas_ = new ImageCanvas( this );
}

TopFrame::~TopFrame()
{
  delete Canvas_;
  Canvas_ = NULL;
}

void TopFrame::OnChar( wxKeyEvent& event )
{
//   long evkey = event.KeyCode();
}

void TopFrame::OnClose( wxCloseEvent& event )
{
  MyGui_->DestroyWindow();
}


