/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include <Matcher2D/RegionMatcher.hh>
#include <Base/Image/Image.hh>
#include <Base/Image/ImageDraw.hh>
#include <Base/Image/ImageIO.hh>
#include <Base/Geometry/HomgPoint2D.hh>
#include <Base/Debug/TimeMeasure.hh>
#include <Filter/Gradient.hh>

#include <iostream>
#include <iomanip>

using namespace BIAS;
using namespace std;

// ./ExampleKLTEdge 157 100 ~/cvs/TestData/tracker_noise/image*.pgm
// ./ExampleKLTEdge 100 157 ~/cvs/TestData/tracker_noise/image*.pgm
// ./ExampleKLTEdge 139 115 ~/cvs/TestData/tracker_diag_noise/image*.pgm
// ./ExampleKLTEdge 116 115 ~/cvs/TestData/tracker_diag_noise/image*.pgm


int main(int argc, char*argv[])
{
  Image<unsigned char> im[2], gradx, grady, gradNWSE, gradNESW, drawim;
  KLT_TYPE p[2][2], error, maxerror=0.02, err, errorsum=0.0;
  unsigned int uip[2], rectsize=7;
  unsigned int maxiter=10; //, numiter=15;
  unsigned int halfwinsize=3;
  unsigned int newim, oldim, res;
  int counter;
  //bool UseErrorAsStopCriterion=true;
  bool debug=true;
  string file;
  bool draw=false;
  MetaData *md;
  AppData appdata;
  HomgPoint2D realpoint;

  RegionMatcher matcher;
  if (debug) matcher.SetDebugLevel(D_REGION_MATCHER_KLT_EDGE);
  Gradient grad;
  TimeMeasure timer;
  TimeMeasure tc;

  if (argc<4){
    cerr << argv[0] << " : <x-coo> <y-coo> <image> <image> ..."<< endl;
    return -5;
  }

  p[0][0]=atof(argv[1]);
  p[0][1]=atof(argv[2]);

  if (ImageIO::Load(argv[3], im[0])!=0){
    BIASERR("error loading image "<<argv[3]);
    return -1;
  }
  
  counter=3;
  newim=0;
  while (counter < argc-1){
    counter++;
    newim=(newim==0)?(1):(0);
    oldim=(newim==0)?(1):(0);
    
    if (ImageIO::Load(argv[counter], im[newim])!=0){
      BIASERR("error loading image "<<argv[counter]);
      return -1;
    } else {
      if (debug) 
        cout << "--------------- newim: " << argv[counter] 
             << "  --- oldim: " << argv[counter-1] << endl;
      md=im[newim].GetMetaData();
      if (md->Find(AppData::MD_HomgPoint2D, "#[HomgPoint2D]", appdata)>=0){
        if (appdata.tag==AppData::MD_USE_ASCII){
          if (debug) cerr << appdata.stag <<" : "<<appdata.sdata<<endl;
          sscanf(appdata.sdata.c_str(), "[ %lf %lf %lf ]",
                 &realpoint[0], &realpoint[1], &realpoint[2]);
        } else {
          memcpy(realpoint.GetData(), appdata.data, appdata.length);
        }
        if (debug) cerr <<"realpoint is at "<<realpoint<<endl;
      } else {
        BIASERR("cannot find HomgPoint2D in meta data");
      }
    }
    
    if (p[oldim][0]<=halfwinsize+3 ||
        p[oldim][1]<=halfwinsize+3 ||
        p[oldim][0]+halfwinsize+3>=im[oldim].GetWidth()-1 ||
        p[oldim][1]+halfwinsize+3>=im[oldim].GetHeight()-1){
      cout << "point ("<<p[oldim][0]<<", "<<p[oldim][1]<<") too close to "
           <<"image border, stopping"<<endl;
      break;
    }
      

    grad.SobelWE3(im[oldim], gradx);
    grad.SobelNS3(im[oldim], grady);
    grad.SobelNWSE3(im[oldim], gradNWSE);
    grad.SobelNESW3(im[oldim], gradNESW);
    if (debug){
      ImageIO::Write("gradWE.mip",gradx);
      ImageIO::Write("gradNS.mip",grady);
      ImageIO::Write("gradNWSE.mip",gradNWSE);
      ImageIO::Write("gradNESW.mip",gradNESW);
    }
    /*
      if (!UseErrorAsStopCriterion){
      if (debug) cerr <<"doing "<<numiter<<" iterations"<<endl;
      timer.Start();
      tc.Start();
      res = matcher.KLTEdge(p[oldim], p[oldim], im[oldim].GetImageDataArray(),
      im[newim].GetImageDataArray(), 
      gradx.GetImageDataArray(),
      grady.GetImageDataArray(), 
      gradNWSE.GetImageDataArray(),
      gradNESW.GetImageDataArray(), 
      im[oldim].GetWidth(), im[oldim].GetHeight(), 
      numiter, p[newim], error);
      tc.Stop();
      timer.Stop();

      } else {
    */
    if (debug) 
      cerr <<"using maxerror "<<maxerror<<" as stopping criterion"<<endl;
    timer.Start();
    tc.Start();
	//KLTEdge no such function
	/*
    res = matcher.KLTEdge(p[oldim], p[oldim], im[oldim].GetImageDataArray(),
                          im[newim].GetImageDataArray(), 
                          gradx.GetImageDataArray(),
                          grady.GetImageDataArray(), 
                          gradNWSE.GetImageDataArray(),
                          gradNESW.GetImageDataArray(), 
                          im[oldim].GetWidth(), im[oldim].GetHeight(), 
                          maxiter, maxerror, p[newim], error);
    */
	tc.Stop();
    timer.Stop();
    //    }

    tc.Print();
    tc.Reset();

    if (res!=0){
      BIASERR("error matching");
      break;
    }
    err=sqrt((p[newim][0]-realpoint[0])*(p[newim][0]-realpoint[0])+
             (p[newim][1]-realpoint[1])*(p[newim][1]-realpoint[1]));
    errorsum+=err;

    if (draw){
      uip[0]=(unsigned int)rint(p[newim][0]);
      uip[1]=(unsigned int)rint(p[newim][1]);
      drawim=im[newim];
      ImageDraw<unsigned char>::RectangleCenter(drawim, uip, rectsize);
 
      file=argv[counter];
      file+=".tracked";
      ImageIO::Write(file, drawim);
    }
    cout << setw(2) << counter-3 << " : (" << p[oldim][0]<<", "<<p[oldim][1]
         << ") -> ("<< p[newim][0]<<", "<<p[newim][1]<<")  klt-error: "<<error
         <<"   real: ("<<realpoint[0]<<", "<<realpoint[1]
         <<")   dist-err: "<<err<<endl;  

  }
  cerr << "errorsum over "<<counter-3<<" images: "<<errorsum<<endl
       << "  mean error "<<errorsum/(double)(counter-3)<<endl;

  timer.Print();
  cerr << "KLT trackin over "<<argc-3<<" images took "<<timer.GetRealTime()
       <<" us"
       << "\n  this is equal to "<<timer.GetRealTime()/double(argc-3)
       <<" us per image and corner" << endl;
  return 0;
}

