/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "TrackerGaussianWeightedKLT.hh"
#include "TrackerSimpleKLT.cpp"
  //#include <Filter/Gradient.hh>

using namespace BIAS;
using namespace std;


//////////////////////////////////////////////////////////////////////////
//                 instantiation
//////////////////////////////////////////////////////////////////////////

template class TrackerGaussianWeightedKLT<unsigned char>; 
template class TrackerGaussianWeightedKLT<float>; 

// fill in instances as required
#ifdef BUILD_IMAGE_INT
template class TrackerGaussianWeightedKLT<int>; 
#endif
#ifdef BUILD_IMAGE_CHAR
template class TrackerGaussianWeightedKLT<char>; 
#endif
#ifdef BUILD_IMAGE_SHORT
template class TrackerGaussianWeightedKLT<short>; 
#endif
#ifdef BUILD_IMAGE_USHORT
template class TrackerGaussianWeightedKLT<unsigned short>; 
#endif
#ifdef BUILD_IMAGE_UINT
template class TrackerGaussianWeightedKLT<unsigned int>; 
#endif
#ifdef BUILD_IMAGE_DOUBLE
#endif


//////////////////////////////////////////////////////////////////////////
//                 implementation
//////////////////////////////////////////////////////////////////////////


template <class StorageType>
TrackerGaussianWeightedKLT<StorageType>::
TrackerGaussianWeightedKLT() 
  : TrackerSimpleKLT<StorageType>()
{}

template <class StorageType>
TrackerGaussianWeightedKLT<StorageType>::
TrackerGaussianWeightedKLT(Param& para, const std::string& postfix) 
  : TrackerSimpleKLT<StorageType>(para, postfix)
{}

template <class StorageType>
TrackerGaussianWeightedKLT<StorageType>::
TrackerGaussianWeightedKLT(Param& para, const int grp, 
                           const std::string& postfix) 
  : TrackerSimpleKLT<StorageType>(para, grp, postfix)
{}

template <class StorageType>
TrackerGaussianWeightedKLT<StorageType>::~TrackerGaussianWeightedKLT()
{}

template <class StorageType>
void TrackerGaussianWeightedKLT<StorageType>::
Init(Image<StorageType>& im1, Image<StorageType>& im2, 
     Image<StorageType>& gradx1, Image<StorageType>& grady1, 
     Image<StorageType>& gradx2, Image<StorageType>& grady2)
{
  bool resize=false;
  if (*this->_HalfWinSize != this->_LastHalfWinSize){
     resize=true;
  }
  TrackerSimpleKLT<StorageType>::Init(im1, im2, gradx1, grady1, gradx2,grady2);
  if (resize){
    CalcGauss_();
  }
}

template <class StorageType>
void TrackerGaussianWeightedKLT<StorageType>::
Track(std::vector<HomgPoint2D>& p1, std::vector<HomgPoint2D>& p2, 
      std::vector<HomgPoint2D>& p2tracked, std::vector<KLT_TYPE>& error, 
      std::vector<int>& numiter, std::vector<KLT_TYPE>& residui, 
      std::vector<int>& res)
{
  const unsigned num=p1.size();
  BIASASSERT(num==p2.size() && num==p2tracked.size() && num==error.size());
  BIASASSERT(num==numiter.size() && num==res.size());

  for (unsigned i=0; i<num; i++){
    if (p1[i][2]==1.0){
#ifdef BIAS_DEBUG
      BIASASSERT(p2[i].IsHomogenized());
#endif    
      res[i]=Track_(p1[i].GetData(), p2[i].GetData(), p2tracked[i].GetData(), 
                    error[i], numiter[i], residui[i]);
    } else {
      res[i] = TRACKER_IGNOREPOINT;
    }
  }
}

template <class StorageType>
int TrackerGaussianWeightedKLT<StorageType>::
Track(HomgPoint2D& p1, HomgPoint2D& p2, HomgPoint2D& result, 
      KLT_TYPE& error, int &iter, KLT_TYPE& residuum)
{
#ifdef BIAS_DEBUG
  BIASASSERT(p1.IsHomogenized() && p2.IsHomogenized());
#endif  
  return Track_(p1.GetData(), p2.GetData(), result.GetData(), error, iter, 
                residuum);
}

template <class StorageType>
void TrackerGaussianWeightedKLT<StorageType>::
Init(StorageType **ida1, StorageType **ida2, StorageType **gradx1, 
     StorageType **grady1, StorageType **gradx2, StorageType **grady2, 
     unsigned int width, unsigned int height, unsigned gradient_half_mask)
{
  bool resize=false;
  if (*this->_HalfWinSize != this->_LastHalfWinSize){
     resize=true;
  }
  TrackerSimpleKLT<StorageType>::Init(ida1, ida2, gradx1, grady1, gradx2, 
                                      grady2, width, height, 
                                      gradient_half_mask);
  if (resize){
    CalcGauss_();
  }
}

template <class StorageType>
int TrackerGaussianWeightedKLT<StorageType>::
Track(KLT_TYPE p1[2], KLT_TYPE p2[2], KLT_TYPE result[2], 
      KLT_TYPE& error, int &iter, KLT_TYPE& residuum)
{
  return Track_(p1, p2, result, error, iter, residuum);
}

template <class StorageType>
int TrackerGaussianWeightedKLT<StorageType>::
Track_(KLT_TYPE p1[2], KLT_TYPE p2[2], KLT_TYPE result[2], 
       KLT_TYPE& error, int &iter, KLT_TYPE& residuum)
{
  if (*this->_HalfWinSize != this->_LastHalfWinSize){
    _Resize(*this->_HalfWinSize);
    CalcGauss_();
  }
  BIASCDOUT(D_TRACKERB_KLT,"p1: " << p1[0] << ", " << p1[1] 
            << "\tp2: " <<p2[0] << ", " << p2[1]<<endl);

  int res=TRACKER_SUCCESS;
  int rh, ch;
  
  register KLT_TYPE dI, gxx, gxy, gyy, ex, ey, dx, dy, det, idet;
  KLT_TYPE lsterror=FLT_MAX;
  const int hws=*this->_HalfWinSize;
  const int winsize=this->_WinSize;
  const int maxiter=*this->_MaxIterations;
  const KLT_TYPE maxerr=*this->_MaxError;
  const KLT_TYPE max_residuum=*this->_MaxResiduum;

  if (p1[0]<this->_minx || p1[0]>=this->_maxx || p1[1]<this->_miny 
      || p1[1]>=this->_maxy || p2[0]<this->_minx || p2[0]>=this->_maxx 
      || p2[1]<this->_miny || p2[1]>=this->_maxy ){
    BIASCDOUT(D_TRACKERB_KLT, "points do not lie in ROI, returning -4\n");
    return TRACKER_OUTOFROI ;
  }  

  iter=0;

  result[0]=p2[0];
  result[1]=p2[1];
  //  MyBilinearRegion(_ida1,_gradx1,_grady1,p1[0],p1[1],hws,_bl1,_gx1,_gy1);
  this->_MyBilinearRegion1(p1[0],p1[1],hws);
  do {
    gxx=gxy=gyy=ex=ey=0.0;
    //     MyBilinearRegion(_ida2,_gradx2,_grady2,result[0],result[1],hws,
    //                      _bl2,_gx2,_gy2);
    this->_MyBilinearRegion2(result[0],result[1],hws);
    // initialize G and e
    for (rh=0; rh<winsize; rh++){
      for (ch=0; ch<winsize; ch++){
        this->_gsx[rh][ch] = 
          _gauss[rh][ch] * (this->_gx1[rh][ch] + this->_gx2[rh][ch]);
        this->_gsy[rh][ch] = 
          _gauss[rh][ch] * (this->_gy1[rh][ch] + this->_gy2[rh][ch]);
        gxx += this->_gsx[rh][ch]*this->_gsx[rh][ch];
        gyy += this->_gsy[rh][ch]*this->_gsy[rh][ch];
        gxy += this->_gsx[rh][ch]*this->_gsy[rh][ch];
        dI = _gauss[rh][ch] * (this->_bl1[rh][ch] - this->_bl2[rh][ch]);
        ex += this->_gsx[rh][ch]*dI;
        ey += this->_gsy[rh][ch]*dI;
      } 
    }
    BIASCDOUT(D_TRACKERB_KLT, "gxx: "<<gxx<<" gxy: "<<gxy
              <<" gyy: "<<gyy<<" ex: "<<ex<<" ey: "<<ey<<endl);
    det = gxx*gyy-gxy*gxy;
    if (fabs(det)<1e-8) {
      BIASCDOUT(D_TRACKERB_KLT, 
                "unable to invert G, det: "<<det<<endl);

      res = TRACKER_NOSTRUCTURE;
      break;
    }
    idet=1.0/det;
    dx = (gyy*ex - gxy*ey)*idet; 
    dy = (gxx*ey - gxy*ex)*idet; 

    BIASCDOUT(D_TRACKERB_KLT, "det: "<<det<<" idet: "<<idet
              <<" dx: "<<dx<<" dy: "<<dy<<endl);
    
    //error = sqrt(dx*dx + dy*dy);
    error = (fabs(dx)>fabs(dy))?fabs(dx):fabs(dy);
    /*
      if (error>lsterror){
      BIASCDOUT(D_TRACKERB_KLT, "error increased ("
      <<lsterror<<" -> "<<error
      <<"), stopping iteration and returning 1\n");
      res = TRACKER_ERRORINCREASED;
      error=lsterror;
      break;
      }
    */
    lsterror=error;

    result[0] += dx;
    result[1] += dy;

    BIASCDOUT(D_TRACKERB_KLT,"disp after "<<iter<<" iteration: "
              <<result[0]-p2[0]<<", "<<result[1]-p2[1]<<" error: "<< error
              << "  ("<<result[0]<<", "<<result[1]<<")\n");

    iter++;
    
    if (result[0]<this->_minx || result[0]>=this->_maxx ||
        result[1]<this->_miny || result[1]>=this->_maxy){
      BIASCDOUT(D_TRACKERB_KLT, 
                "feature slid out of image, returning -2\n");
      res = TRACKER_OUTOFIMAGE;
      break;
    }
  } while (error>maxerr && iter<maxiter);
    
  if (res==0 && error>maxerr) {
    res = TRACKER_TOOMANYITER;
    BIASCDOUT(D_TRACKERB_KLT,"error > maxerror : returning -3\n");
  }

  if (res==0){
    const int wsize= winsize*winsize;
    KLT_TYPE *f1=this->_bl1.GetData(), *f2=this->_bl2.GetData(), *end=f1+wsize;
    residuum=0.0;
    while (f1<end){
      residuum += fabs(*f1++ - *f2++);
    }
    residuum/=(KLT_TYPE)wsize;
    if (residuum>max_residuum){
      res = TRACKER_MEANABSDIF;
      BIASCDOUT(D_TRACKERB_KLT, "residuuum = "<<residuum<<" > "<<max_residuum
                <<" returning -5");
    }
  }

  return res;
}


template <class StorageType>
void TrackerGaussianWeightedKLT<StorageType>::
CalcGauss_()
{
  _gauss.newsize(this->_WinSize, this->_WinSize);
  // now calculate the gaussian weight matrix
  double sigma = ((double)this->_WinSize)/6.0;
  double fac = 1.0 / (2.0 * sigma * sigma);
  double sum=0.0, tmp;
  register int i, j, x, y, xp, xm, yp, ym;
  const int hws=*this->_HalfWinSize;
  for (i= hws; i >= 0; i--){
    for (j= hws; j >= 0; j--){
      tmp=exp(-(double)(i*i+j*j)*fac);
      yp=hws+i;
      ym=hws-i;
      xp=hws+j;
      xm=hws-j;
      _gauss[ym][xm] =_gauss[yp][xm] = _gauss[ym][xp] = _gauss[yp][xp] = 
        (KLT_TYPE)(tmp);
    }
  }
    
  for (y=0; y<this->_WinSize; y++){
    for (x=0; x<this->_WinSize; x++){
      sum+=_gauss[y][x];
    } 
  }
  BIASCDOUT(D_TRACKERB_INIT, "_gauss: "<<_gauss<<endl);
  BIASCDOUT(D_TRACKERB_INIT, "_gauss sum before normalization: "<<sum<<endl);

  //normalization
  sum=1.0/sum; // division is slow
  for (i= hws; i >= 0; i--){
    for (j= hws; j >= 0; j--){
      yp=hws+i;
      ym=hws-i;
      xp=hws+j;
      xm=hws-j;
      tmp=_gauss[ym][xm];
      _gauss[ym][xm] = _gauss[yp][xm] = _gauss[ym][xp] =
        _gauss[yp][xp] = tmp * sum;
    }
  }
  BIASCDOUT(D_TRACKERB_INIT, "_gauss: "<<_gauss<<endl);
#ifdef BIAS_DEBUG
  sum=0.0;
  for (y=0; y<this->_WinSize; y++){
    for (x=0; x<this->_WinSize; x++){
      sum+=_gauss[y][x];
    } 
  }
  BIASCDOUT(D_TRACKERB_INIT,"_gauss sum after normalization: "<<sum<<endl);
#endif  
}

