/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include <Base/Image/ImageBase.hh>
#include <Base/Image/ImageIO.hh>
#include <stdio.h>


using namespace BIAS;
using namespace std;

void usage()
{
  cout << "Usage: biassetuid [options] files"<<endl;
  cout <<" Options: "<< endl
       <<" -f         : force setting of a new uid"<<endl
       << endl;
  cout <<"This program opens the image files given by the command line"
       <<", checks for a valid id and assigns a new one if there is no."<<endl
       <<"Returns: 0 on success, 1 if errors occured."<<endl<<endl;
}

int DetermineFiletype(const string &fn, ImageIO::TFileFormat &thetype)
{
  int EndOfPrefix;
  string suffix;
  
  EndOfPrefix = fn.rfind('.');
  
  suffix = fn.substr(EndOfPrefix+1,fn.length());
  if (suffix == "pnm") thetype =  ImageIO::FF_ppm;
  else if (suffix == "ppm") thetype =  ImageIO::FF_ppm;
  else if (suffix == "pgm") thetype =  ImageIO::FF_pgm;
  else if (suffix == "mip") thetype =  ImageIO::FF_mip;
  else return -1;

  return 0;
}


/** @class biassetuid
    @ingroup g_tools
    @brief ???
    @author koeser */
int main (int argc, char *argv[])
{
  bool ForceNewUUID = false;
  int i = 1;
  if (argc>1) if (!strcmp("-f",argv[1])) {
    std::cout << "Existing UIDs are replaced."<<endl;
    ForceNewUUID = true;
    i = 2;
  }
  if (i>=argc) { 
    usage();
    //return -1;
	return 0; // for Dart testing 
  }
  
  ImageIO::TFileFormat FileType;
  bool ErrorFlag = false;
  int IOResult = 0;
  BIAS::ImageBase CurImage;

  for (;i<argc;i++) {

    if (DetermineFiletype(argv[i], FileType)!=0) {
      BIASERR("Skipping current file because of unknown/unsuported format: "
              <<argv[i]<<endl<<"UIDs can only be written to mip,pgm,ppm,pnm");
      ErrorFlag = true;
      continue;
    }

    IOResult = ImageIO::Load(argv[i], CurImage);
    if (IOResult!=0) {
      ErrorFlag = true;
      BIASERR("Error reading "<< argv[i]);
      continue;
    }
    if (FileType==ImageIO::FF_mip) {
      IOResult = ImageIO::Write(argv[i], CurImage, false, ForceNewUUID );
    } else {
      IOResult = ImageIO::ExportImage(argv[i], CurImage, FileType, false, 
                                      0, ForceNewUUID);
    }
    if (IOResult!=0) {
      ErrorFlag = true;
      BIASERR("Error writing "<< argv[i]);
      continue;
    }
    cout << "File " << argv[i] << " has UID "<<CurImage.GetUID() << endl;
  }
  if (ErrorFlag) cout << "There were errors." <<endl;
  if (ErrorFlag) return 1;
  return 0;
    
}












